@@ -1,5 +1,5 @@
 /* GDA FireBird Provider
- * Copyright (C) 1998-2002 The GNOME Foundation
+ * Copyright (C) 1998 - 2005 The GNOME Foundation
  *
  * AUTHORS:
  *         Albi Jeronimo <jeronimoalbi@yahoo.com.ar>
@@ -22,10 +22,11 @@
 
 #include <libgda/gda-intl.h>
 #include <libgda/gda-data-model-array.h>
+#include <libgda/gda-command.h>
+#include <glib/gprintf.h>
+#include <string.h>
 #include "gda-firebird-provider.h"
 #include "gda-firebird-recordset.h"
-#include <string.h>
-#include <glib/gprintf.h>
 
 static void 		gda_firebird_provider_class_init (GdaFirebirdProviderClass *klass);
 static void 		gda_firebird_provider_init (GdaFirebirdProvider *provider,
@@ -47,12 +48,12 @@ static const gchar 	*gda_firebird_provider_get_database (GdaServerProvider *prov
 static gboolean		gda_firebird_provider_change_database (GdaServerProvider *provider,
 							       GdaConnection *cnc,
 							       const gchar *name);
-static gboolean 	gda_firebird_provider_create_database (GdaServerProvider *provider,
-							       GdaConnection *cnc,
-							       const gchar *name);
-static gboolean 	gda_firebird_provider_drop_database (GdaServerProvider *provider,
-							     GdaConnection *cnc,
-							     const gchar *name);
+static gboolean 	gda_firebird_provider_create_database_cnc (GdaServerProvider *provider,
+					  		           GdaConnection *cnc,
+							           const gchar *name);
+static gboolean 	gda_firebird_provider_drop_database_cnc (GdaServerProvider *provider,
+								 GdaConnection *cnc,
+								 const gchar *name);
 static GList 		*gda_firebird_provider_execute_command (GdaServerProvider *provider,
 								GdaConnection *cnc,
 								GdaCommand *cmd,
@@ -95,8 +96,8 @@ gda_firebird_provider_class_init (GdaFirebirdProviderClass *klass)
 	provider_class->get_server_version = gda_firebird_provider_get_server_version;
 	provider_class->get_database = gda_firebird_provider_get_database;
 	provider_class->change_database = gda_firebird_provider_change_database;
-	provider_class->create_database = gda_firebird_provider_create_database;
-	provider_class->drop_database = gda_firebird_provider_drop_database;
+	provider_class->create_database_cnc = gda_firebird_provider_create_database_cnc;
+	provider_class->drop_database_cnc = gda_firebird_provider_drop_database_cnc;
 	provider_class->execute_command = gda_firebird_provider_execute_command;
 	provider_class->begin_transaction = gda_firebird_provider_begin_transaction;
 	provider_class->commit_transaction = gda_firebird_provider_commit_transaction;
@@ -132,7 +133,7 @@ gda_firebird_provider_get_type (void)
 			(GBaseInitFunc) NULL,
 			(GBaseFinalizeFunc) NULL,
 			(GClassInitFunc) gda_firebird_provider_class_init,
-			NULL, 
+			NULL,
 			NULL,
 			sizeof (GdaFirebirdProvider),
 			0,
@@ -154,6 +155,7 @@ gda_firebird_provider_new (void)
 	return GDA_SERVER_PROVIDER (fprv);
 }
 
+
 /*
  *  fb_server_get_version
  *
@@ -173,7 +175,7 @@ fb_server_get_version (GdaFirebirdConnection *fcnc)
 	};
 	
 	/* Try to get datbase version */
-	if (! isc_database_info (fcnc->status, &(fcnc->handle), sizeof (fdb_info), fdb_info, 
+	if (! isc_database_info (fcnc->status, &(fcnc->handle), sizeof (fdb_info), fdb_info,
 				 sizeof (buffer), buffer)) {
 		p_buffer = buffer;
 		if (*p_buffer != isc_info_end) {
@@ -200,7 +202,7 @@ fb_get_types (GdaConnection *cnc,
 	      GdaParameterList *params)
 {
 	GdaDataModelArray *recset;
-	GList *value_list = NULL;	
+	GList *value_list;
 	gint i;
 	struct {
 		const gchar *name;
@@ -228,17 +230,20 @@ fb_get_types (GdaConnection *cnc,
 	recset = (GdaDataModelArray *) gda_data_model_array_new (4);
 	gda_data_model_set_column_title (GDA_DATA_MODEL (recset), 0, _("Type"));
 	gda_data_model_set_column_title (GDA_DATA_MODEL (recset), 1, _("Owner"));
+	gda_data_model_set_column_title (GDA_DATA_MODEL (recset), 2, _("Comments"));
 	gda_data_model_set_column_title (GDA_DATA_MODEL (recset), 3, _("GDA type"));
-
+	
 	/* Fill the recordset with each of the values from the array defined above */
-	for (i = 0; i < sizeof (types) / sizeof (types[0]); i++) {
+	for (i = 0; i < (sizeof (types) / sizeof (types[0])); i++) {
+		value_list = NULL;
+		
 		value_list = g_list_append (value_list, gda_value_new_string (types[i].name));
 		value_list = g_list_append (value_list, gda_value_new_string (types[i].owner));
 		value_list = g_list_append (value_list, gda_value_new_string (types[i].comments));
-		value_list = g_list_append (value_list, gda_value_new_type (types[i].type));
+		value_list = g_list_append (value_list, gda_value_new_gdatype (types[i].type));
 
 		/* Add values to row */
-		gda_data_model_append_row (GDA_DATA_MODEL (recset), value_list);
+		gda_data_model_append_values (GDA_DATA_MODEL (recset), value_list);
 
 		/* Free temporary list of values */
 		g_list_foreach (value_list, (GFunc) gda_value_free, NULL);
@@ -254,7 +259,7 @@ fb_get_types (GdaConnection *cnc,
  *
  *  @show_views: If TRUE only return views, else return tables
  *
- *  Function looks for the following parameters in list: 
+ *  Function looks for the following parameters in list:
  *    systables (gboolean)
  *
  *  Returns: A GdaDataModel containing Firebird's tables/views
@@ -308,7 +313,7 @@ fb_get_tables (GdaConnection *cnc,
 		/* Execute statement */
 		command = gda_command_new (sql, GDA_COMMAND_TYPE_SQL, GDA_COMMAND_OPTION_STOP_ON_ERRORS);
 		gda_command_set_transaction (command, transaction);		
-		reclist = gda_connection_execute_command (cnc, command, NULL);
+		reclist = gda_connection_execute_command (cnc, command, NULL, NULL);
 
 		if (reclist) {
 			recset = gda_data_model_array_new (4);
@@ -323,20 +328,20 @@ fb_get_tables (GdaConnection *cnc,
 				row = (GdaRow *) gda_data_model_get_row (GDA_DATA_MODEL (reclist->data), i);
 				value = gda_row_get_value (row, 0);
 				value_list = g_list_append (
-						value_list, 
+						value_list,
 						gda_value_new_string ((gchar *) gda_value_get_string (value)));
 
 				/* Get owner of table */
 				value = gda_row_get_value (row, 1);
 				value_list = g_list_append (
-						value_list, 
+						value_list,
 						gda_value_new_string ((gchar *) gda_value_get_string (value)));
 				
 				/* FIXME: Set correct values */
 				value_list = g_list_append (value_list, gda_value_new_string (""));
 				value_list = g_list_append (value_list, gda_value_new_string (""));
 
-				gda_data_model_append_row (recset, value_list);
+				gda_data_model_append_values (recset, value_list);
 				g_list_foreach (value_list, (GFunc) gda_value_free, NULL);
 				g_list_free (value_list);
 			}
@@ -358,7 +363,7 @@ fb_get_tables (GdaConnection *cnc,
 /*
  *  fb_type_name_to_gda_type
  *
- *  Function convert a string containing the name of Firebird's 
+ *  Function convert a string containing the name of Firebird's
  *  data type to it's corresponding gda data type.
  *
  *  Returns: The GdaValueType corresponding to @name Firebird's
@@ -395,14 +400,14 @@ fb_type_name_to_gda_type (const gchar *name)
 
 
 /*
- *  fb_set_index_field_metadata
+ *  fb_set_index_field_metad
  *
  *  Sets primary key and unique key properties for fields
  *  in @recset.
  *  This function is called from #fb_get_fields_metadata.
  */
 static void
-fb_set_index_field_metadata (GdaConnection *cnc, 
+fb_set_index_field_metadata (GdaConnection *cnc,
 			     GdaDataModel *recset,
 			     const gchar *table_name)
 {
@@ -421,14 +426,14 @@ fb_set_index_field_metadata (GdaConnection *cnc,
 		sql = g_strdup_printf (
 				"SELECT A.RDB$FIELD_NAME, I.RDB$UNIQUE_FLAG, B.RDB$CONSTRAINT_TYPE "
 				"FROM RDB$INDEX_SEGMENTS A, RDB$RELATION_CONSTRAINTS B, RDB$INDICES I "
-				"WHERE ((B.RDB$CONSTRAINT_TYPE = 'PRIMARY KEY') OR (I.RDB$UNIQUE_FLAG = 1)) "
-				"AND (A.RDB$INDEX_NAME = B.RDB$INDEX_NAME) AND (B.RDB$RELATION_NAME = '%s') "
-				"AND (A.RDB$INDEX_NAME = I.RDB$INDEX_NAME) "
+				"WHERE ((B.RDB$CONSTRAINT_TYPE = 'PRIMARY KEY') "
+				"OR (I.RDB$UNIQUE_FLAG = 1)) AND (A.RDB$INDEX_NAME = B.RDB$INDEX_NAME) "
+				"AND (B.RDB$RELATION_NAME = '%s') AND (A.RDB$INDEX_NAME = I.RDB$INDEX_NAME) "
 				"ORDER BY A.RDB$FIELD_POSITION",
 				table_name);
 		command = gda_command_new (sql, GDA_COMMAND_TYPE_SQL, GDA_COMMAND_OPTION_STOP_ON_ERRORS);
 		gda_command_set_transaction (command, transaction);
-		reclist = gda_connection_execute_command (cnc, command, NULL);
+		reclist = gda_connection_execute_command (cnc, command, NULL, NULL);
 		if (reclist) {
 			gda_data_model_freeze (recset);
 			
@@ -436,17 +441,14 @@ fb_set_index_field_metadata (GdaConnection *cnc,
 			for (i = 0; i < gda_data_model_get_n_rows (GDA_DATA_MODEL (reclist->data)); i++) {
 				value = (GdaValue *) gda_data_model_get_value_at (GDA_DATA_MODEL (reclist->data), 0, i);
 				field_name = (gchar *) gda_value_get_string (value);
-				j = 0;
-				recset_value = (GdaValue *) gda_data_model_get_value_at (recset, 0, j);
-				recset_field_name = (gchar *) gda_value_get_string (recset_value);
-				
+				j = -1;
+
 				/* Find column in @recset by name */
-				for (j = 0; (strcmp (recset_field_name, field_name)) 
-					    && (j < gda_data_model_get_n_rows (recset)); /*nop*/) {
+				do {
+					j++;
 					recset_value = (GdaValue *) gda_data_model_get_value_at (recset, 0, j);
 					recset_field_name = (gchar *) gda_value_get_string (recset_value);
-					j++;
-				}
+				} while (strcmp (recset_field_name, field_name) && (j < gda_data_model_get_n_rows (recset)));
 				
 				/* Set recset values */
 				if (!strcmp (recset_field_name, field_name)) {
@@ -491,7 +493,7 @@ fb_set_index_field_metadata (GdaConnection *cnc,
 
 
 /*
- *  fb_set_field_metadata
+ *  fb_set_field_metad
  *
  *  Fill a list with metadata values from @row.
  *  This function is called from #fb_get_fields_metadata.
@@ -503,7 +505,7 @@ fb_set_field_metadata (GdaRow *row)
 {
 	GList *value_list = NULL;
 	GdaValue *value;
-	gchar *the_value;
+	gchar *the_value, *tmp_value;
 	gshort scale, short_value;
 
 	g_return_val_if_fail (row != NULL, NULL);
@@ -515,29 +517,33 @@ fb_set_field_metadata (GdaRow *row)
 	
 	/* Get Scale (number of decimals) */
 	value = gda_row_get_value (row, 4);
-	if (!gda_value_is_null (value)) 
+	if (!gda_value_is_null (value))
 		scale = gda_value_get_smallint (value);
 	else
 		scale = 0;
 	
 	/* Set data type */
 	value = gda_row_get_value (row, 1);
-	the_value = (gchar *) gda_value_get_string (value);
-	the_value = g_strchomp (the_value);
-
+	tmp_value = (gchar *) gda_value_get_string (value);
+	tmp_value = g_strchomp (tmp_value);
+	the_value = g_ascii_strdown (tmp_value, -1);
+	g_free (tmp_value);
+	
 	/* Set correct value name */
-	if (!strcmp (the_value, "LONG") && (scale < 0))
-		value_list = g_list_append (value_list, gda_value_new_string ("DECIMAL"));
-	else if (!strcmp (the_value, "LONG"))
-		value_list = g_list_append (value_list, gda_value_new_string ("INTEGER"));
-	else if (!strcmp (the_value, "INT64"))
-		value_list = g_list_append (value_list, gda_value_new_string ("NUMERIC"));
-	else if (!strcmp (the_value, "VARYING"))
-		value_list = g_list_append (value_list, gda_value_new_string ("VARCHAR"));
-	else if (!strcmp (the_value, "TEXT"))
-		value_list = g_list_append (value_list, gda_value_new_string ("CHAR"));
-	else if (!strcmp (the_value, "SHORT"))
-		value_list = g_list_append (value_list, gda_value_new_string ("SMALLINT"));
+	if (!strcmp (the_value, "long") && (scale < 0))
+		value_list = g_list_append (value_list, gda_value_new_string ("decimal"));
+	else if (!strcmp (the_value, "long"))
+		value_list = g_list_append (value_list, gda_value_new_string ("integer"));
+	else if (!strcmp (the_value, "int64"))
+		value_list = g_list_append (value_list, gda_value_new_string ("numeric"));
+	else if (!strcmp (the_value, "varying"))
+		value_list = g_list_append (value_list, gda_value_new_string ("varchar"));
+	else if (!strcmp (the_value, "text"))
+		value_list = g_list_append (value_list, gda_value_new_string ("char"));
+	else if (!strcmp (the_value, "short"))
+		value_list = g_list_append (value_list, gda_value_new_string ("smallint"));
+	else if (!strcmp (the_value, "double"))
+		value_list = g_list_append (value_list, gda_value_new_string ("double precision"));
 	else
 		value_list = g_list_append (value_list, gda_value_new_string (the_value));
 
@@ -582,12 +588,12 @@ fb_set_field_metadata (GdaRow *row)
 
 
 /*
- *  fb_get_fields_metadata
+ *  fb_get_fields_metad
  *
  *  @params must contain a parameter named "name" with
  *  name of table to describe. ... :-)
  *
- *  Returns: A GdaDataModel containing table metadata
+ *  Returns: A GdaDataModel containing table metad
  */
 static GdaDataModel *
 fb_get_fields_metadata (GdaConnection *cnc,
@@ -622,14 +628,14 @@ fb_get_fields_metadata (GdaConnection *cnc,
 	
 	par = gda_parameter_list_find (params, "name");
 	if (!par) {
-		gda_connection_add_error_string (cnc, 
+		gda_connection_add_event_string (cnc,
 				_("Table name is needed but none specified in parameter list"));
 		return NULL;
 	}
 
 	table_name = gda_value_get_string ((GdaValue *) gda_parameter_get_value (par));
 	if (!table_name) {
-		gda_connection_add_error_string (cnc,
+		gda_connection_add_event_string (cnc,
 				_("Table name is needed but none specified in parameter list"));
 		return NULL;
 	}
@@ -646,7 +652,7 @@ fb_get_fields_metadata (GdaConnection *cnc,
 				table_name);
 		command = gda_command_new (sql, GDA_COMMAND_TYPE_SQL, GDA_COMMAND_OPTION_STOP_ON_ERRORS);
 		gda_command_set_transaction (command, transaction);
-		reclist = gda_connection_execute_command (cnc, command, NULL);
+		reclist = gda_connection_execute_command (cnc, command, NULL, NULL);
 		if (reclist) {
 			/* Create and fill recordset to be returned */
 			recset = (GdaDataModelArray *) gda_data_model_array_new (9);
@@ -660,7 +666,7 @@ fb_get_fields_metadata (GdaConnection *cnc,
 				
 				/* Set field metdata for row, then append to recordset */
 				value_list = fb_set_field_metadata (row);
-				gda_data_model_append_row (GDA_DATA_MODEL (recset), value_list);
+				gda_data_model_append_values (GDA_DATA_MODEL (recset), value_list);
 
 				/* Free temp list of values */
 				g_list_foreach (value_list, (GFunc) gda_value_free, NULL);
@@ -681,6 +687,92 @@ fb_get_fields_metadata (GdaConnection *cnc,
 	return GDA_DATA_MODEL (recset);
 }
 
+static void
+fb_add_aggregate_row (GdaDataModelArray *recset,
+		      const gchar *str,
+		      const gchar *comments)
+{
+	GList *list;
+
+	g_return_if_fail (GDA_IS_DATA_MODEL_ARRAY (recset));
+
+	/* 1st the name */
+	list = g_list_append (NULL, gda_value_new_string (str));
+	/* 2nd the unique id */
+	list = g_list_append (list, gda_value_new_string (str));
+	/* 3rd the owner */
+	list = g_list_append (list, gda_value_new_string (NULL));
+	/* 4th the comments */
+	list = g_list_append (list, gda_value_new_string (comments));
+	/* 5th the return type */
+	list = g_list_append (list, gda_value_new_string (_("UNKNOWN")));
+	/* 6th the argument type */
+	list = g_list_append (list, gda_value_new_string (_("UNKNOWN")));
+	/* 7th the SQL definition */
+	list = g_list_append (list, gda_value_new_string (NULL));
+
+	gda_data_model_append_values (GDA_DATA_MODEL (recset), list);
+
+	g_list_foreach (list, (GFunc) gda_value_free, NULL);
+	g_list_free (list);
+}
+
+static GdaDataModel *
+fb_get_aggregates (GdaConnection *cnc,
+		   GdaParameterList *params)
+{
+	GdaDataModelArray *recset;
+
+	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
+
+	/* Create recordset */
+	recset = (GdaDataModelArray *) gda_data_model_array_new (7);
+	gda_data_model_set_column_title (GDA_DATA_MODEL (recset), 0, _("Name"));
+	gda_data_model_set_column_title (GDA_DATA_MODEL (recset), 1, _("ID"));
+	gda_data_model_set_column_title (GDA_DATA_MODEL (recset), 2, _("Owner"));
+	gda_data_model_set_column_title (GDA_DATA_MODEL (recset), 3, _("Comments"));
+	gda_data_model_set_column_title (GDA_DATA_MODEL (recset), 4, _("Return type"));
+	gda_data_model_set_column_title (GDA_DATA_MODEL (recset), 5, _("Args types"));
+	gda_data_model_set_column_title (GDA_DATA_MODEL (recset), 6, _("SQL"));
+
+	/* Fill recordset */
+	/*FIXME: Add user defined fuctions (Stored in systables) and set parameters */
+	fb_add_aggregate_row (recset, "abs", "comments");
+	fb_add_aggregate_row (recset, "acos","comments");
+	fb_add_aggregate_row (recset, "ascii_char", "comments");
+	fb_add_aggregate_row (recset, "ascii_val", "comments");
+	fb_add_aggregate_row (recset, "asin", "comments");
+	fb_add_aggregate_row (recset, "atan", "comments");
+	fb_add_aggregate_row (recset, "atan2", "comments");
+	fb_add_aggregate_row (recset, "bin_and", "comments");
+	fb_add_aggregate_row (recset, "bin_or", "comments");
+	fb_add_aggregate_row (recset, "bin_xor", "comments");
+	fb_add_aggregate_row (recset, "ceiling", "comments");
+	fb_add_aggregate_row (recset, "cos", "comments");
+	fb_add_aggregate_row (recset, "cosh", "comments");
+	fb_add_aggregate_row (recset, "cot", "comments");
+	fb_add_aggregate_row (recset, "div", "comments");
+	fb_add_aggregate_row (recset, "floor", "comments");
+	fb_add_aggregate_row (recset, "ln", "comments");
+	fb_add_aggregate_row (recset, "log", "comments");
+	fb_add_aggregate_row (recset, "log10", "comments");
+	fb_add_aggregate_row (recset, "lower", "comments");
+	fb_add_aggregate_row (recset, "ltrim", "comments");
+	fb_add_aggregate_row (recset, "mod", "comments");
+	fb_add_aggregate_row (recset, "pi", "comments");
+	fb_add_aggregate_row (recset, "rand", "comments");
+	fb_add_aggregate_row (recset, "rtrim", "comments");
+	fb_add_aggregate_row (recset, "sign", "comments");
+	fb_add_aggregate_row (recset, "sin", "comments");
+	fb_add_aggregate_row (recset, "sinh", "comments");
+	fb_add_aggregate_row (recset, "sqrt", "comments");
+	fb_add_aggregate_row (recset, "strlen", "comments");
+	fb_add_aggregate_row (recset, "substr", "comments");
+	fb_add_aggregate_row (recset, "tan", "comments");
+	fb_add_aggregate_row (recset, "tanh", "comments");	
+
+	return GDA_DATA_MODEL (recset);
+}
 
 /*
  *  fb_sqlerror_get_description
@@ -694,8 +786,8 @@ fb_sqlerror_get_description (GdaFirebirdConnection *fcnc,
 	gchar *tmp_msg;
 	char buffer[512];
 	short sql_code;
-	long * ptr_status_vector = fcnc->status;                                                                                                    
-                                                                                                                            
+	long * ptr_status_vector = fcnc->status;
+
 	/* Get error message */
 	isc_interprete (buffer, &ptr_status_vector);
 	tmp_msg = g_strdup ((gchar *) buffer);
@@ -703,38 +795,38 @@ fb_sqlerror_get_description (GdaFirebirdConnection *fcnc,
 	/* Look for more messages */
 	while (isc_interprete (buffer, &ptr_status_vector))
 		tmp_msg = g_strconcat (tmp_msg, "\n", (gchar *) buffer, NULL);
-                                                                                                                            
+
 	if (statement_type != 0) {
 		sql_code = isc_sqlcode (fcnc->status);
 		isc_sql_interprete (sql_code, buffer, sizeof (buffer));
 		tmp_msg = g_strconcat  (tmp_msg, "\n", (gchar *) buffer, NULL);
 	}
-									                                                                                                                            
+									
 	return tmp_msg;
 }
 
 void
 gda_firebird_connection_make_error (GdaConnection *cnc,
 				    const gint statement_type)
-{                                                                                                                          
-	GdaError *error;
+{
+	GdaConnectionEvent *error;
 	GdaFirebirdConnection *fcnc;
 	gchar *description;
 
 	g_return_if_fail (GDA_IS_CONNECTION (cnc));
-                                                                                                                            
+
 	fcnc = g_object_get_data (G_OBJECT (cnc), CONNECTION_DATA);
 	if (!fcnc) {
-		gda_connection_add_error_string (cnc, _("Invalid Firebird handle"));
+		gda_connection_add_event_string (cnc, _("Invalid Firebird handle"));
 		return;
 	}
 	
-	error = gda_error_new ();
-	gda_error_set_number (error, isc_sqlcode (fcnc->status));
-	gda_error_set_source (error, "[GDA Firebird]");
+	error = gda_connection_event_new (GDA_CONNECTION_EVENT_ERROR);
+	gda_connection_event_set_code (error, isc_sqlcode (fcnc->status));
+	gda_connection_event_set_source (error, "[GDA Firebird]");
 	description = fb_sqlerror_get_description (fcnc, statement_type);
-	gda_error_set_description (error, description);
-	gda_connection_add_error (cnc, error);	
+	gda_connection_event_set_description (error, description);
+	gda_connection_add_event (cnc, error);	
 	g_free (description);
 }
 																								
@@ -742,7 +834,7 @@ gda_firebird_connection_make_error (GdaConnection *cnc,
 static const gchar *
 gda_firebird_provider_get_version (GdaServerProvider *provider)
 {
-	return VERSION;
+	return PACKAGE_VERSION;
 }
 
 /* open_connection handler for the GdaFirebirdProvider class */
@@ -763,7 +855,7 @@ gda_firebird_provider_open_connection (GdaServerProvider *provider,
 	/* get parameters */
 	fb_db = (gchar *) gda_quark_list_find (params, "DATABASE");
 	if (!fb_db) {
-		gda_connection_add_error_string (cnc, _("No database specified"));
+		gda_connection_add_event_string (cnc, _("No database specified"));
 		return FALSE;
 	}
 
@@ -809,11 +901,11 @@ gda_firebird_provider_open_connection (GdaServerProvider *provider,
 		strcpy (dpb, fb_charset);
 		dpb += strlen (fb_charset);
 	}
-                                                                                                                            
+
 	/* Save dpb length */
 	fcnc->dpb_length = dpb - fcnc->dpb_buffer;
 
-	if (isc_attach_database (fcnc->status, strlen (fb_db), fb_db, &(fcnc->handle), fcnc->dpb_length, 
+	if (isc_attach_database (fcnc->status, strlen (fb_db), fb_db, &(fcnc->handle), fcnc->dpb_length,
 				 fcnc->dpb_buffer)) {
 		gda_firebird_connection_make_error (cnc, 0);
 		g_free (fcnc);
@@ -842,7 +934,7 @@ gda_firebird_provider_close_connection (GdaServerProvider *provider,
 	
 	fcnc = g_object_get_data (G_OBJECT (cnc), CONNECTION_DATA);
 	if (!fcnc) {
-		gda_connection_add_error_string (cnc, _("Invalid Firebird handle"));
+		gda_connection_add_event_string (cnc, _("Invalid Firebird handle"));
 		return FALSE;
 	}
 
@@ -869,7 +961,7 @@ gda_firebird_provider_get_server_version (GdaServerProvider *provider,
 
 	fcnc = g_object_get_data (G_OBJECT (cnc), CONNECTION_DATA);
 	if (!fcnc) {
-		gda_connection_add_error_string (cnc, _("Invalid Firebird handle"));
+		gda_connection_add_event_string (cnc, _("Invalid Firebird handle"));
 		return FALSE;
 	}
 
@@ -888,7 +980,7 @@ gda_firebird_provider_get_database (GdaServerProvider *provider,
 
 	fcnc = g_object_get_data (G_OBJECT (cnc), CONNECTION_DATA);
 	if (!fcnc) {
-		gda_connection_add_error_string (cnc, _("Invalid Firebird handle"));
+		gda_connection_add_event_string (cnc, _("Invalid Firebird handle"));
 		return NULL;
 	}
 
@@ -904,27 +996,11 @@ gda_firebird_provider_change_database (GdaServerProvider *provider,
 	return FALSE;
 }
 
-/* Return Firebird transaction handle */
-static isc_tr_handle *
-gda_firebird_command_get_transaction (GdaCommand *cmd)
-{
-	GdaTransaction *xaction;
-	isc_tr_handle *ftr;
-
-	xaction = gda_command_get_transaction (cmd);
-	if (!GDA_IS_TRANSACTION (xaction))
-		return NULL;
-
-	ftr = g_object_get_data (G_OBJECT (xaction), TRANSACTION_DATA);
-	
-	return ftr;
-}
-
-/* create_database handler for the GdaFirebirdProvider class */
+/* create_database_cnc handler for the GdaFirebirdProvider class */
 static gboolean
-gda_firebird_provider_create_database (GdaServerProvider *provider,
-					GdaConnection *cnc,
-					const gchar *name)
+gda_firebird_provider_create_database_cnc (GdaServerProvider *provider,
+					   GdaConnection *cnc,
+					   const gchar *name)
 {
 	gchar *sql;
 	GdaFirebirdConnection *fcnc;
@@ -940,7 +1016,7 @@ gda_firebird_provider_create_database (GdaServerProvider *provider,
 	
 	fcnc = g_object_get_data (G_OBJECT (cnc), CONNECTION_DATA);
 	if (!fcnc) {
-		gda_connection_add_error_string (cnc, _("Invalid Firebird handle"));
+		gda_connection_add_event_string (cnc, _("Invalid Firebird handle"));
 		return FALSE;
 	}
 	
@@ -959,7 +1035,7 @@ gda_firebird_provider_create_database (GdaServerProvider *provider,
 	
 	/* Valid values = 1024, 2048, 4096 and 8192 */
 	if (!fb_page_size)
-		fb_page_size = "4096"; 
+		fb_page_size = "4096";
 		
 	fb_char_set = (gchar *) gda_quark_list_find (params, "CHARACTER_SET");
 	
@@ -967,7 +1043,7 @@ gda_firebird_provider_create_database (GdaServerProvider *provider,
 		fb_char_set = "NONE";
 	
 	/* Create database */	
-	sql = g_strdup_printf ("CREATE DATABASE '%s' USER '%s' PASSWORD '%s' PAGE_SIZE %s DEFAULT CHARACTER SET %s;", 
+	sql = g_strdup_printf ("CREATE DATABASE '%s' USER '%s' PASSWORD '%s' PAGE_SIZE %s DEFAULT CHARACTER SET %s;",
 				name, fb_user, fb_password, fb_page_size, fb_char_set);
 				
 	/* NOTE: Transaction and Database handles should be NULL in this case */
@@ -979,7 +1055,7 @@ gda_firebird_provider_create_database (GdaServerProvider *provider,
 	gda_quark_list_free (params);
 	
 	/* we need to detach from the newly created database */
-	isc_detach_database (fcnc->status, &(fcnc->handle));
+	isc_detach_database (fcnc->status, &fdb);
 
 	return TRUE;
 }
@@ -1018,20 +1094,39 @@ gda_firebird_provider_run_sql (GList *reclist,
 	return reclist;
 }
 
-/* drop_database handler for the GdaFirebirdProvider class */
+/* drop_database_cnc handler for the GdaFirebirdProvider class */
 static gboolean
-gda_firebird_provider_drop_database (GdaServerProvider *provider,
-				     GdaConnection *cnc,
-				     const gchar *name)
+gda_firebird_provider_drop_database_cnc (GdaServerProvider *provider,
+					 GdaConnection *cnc,
+					 const gchar *name)
 {
+	GdaFirebirdConnection *fcnc;
+	isc_db_handle db_handle = NULL;
+	
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
 	g_return_val_if_fail (name != NULL, FALSE);
+
+	/* Get current firebird connection */
+	fcnc = g_object_get_data (G_OBJECT (cnc), CONNECTION_DATA);
+	if (!fcnc) {
+		gda_connection_add_event_string (cnc, _("Invalid Firebird handle"));
+		return FALSE;
+	}
 	
-	/* Connection must be open for database drop */	
-	if (gda_connection_is_open (cnc))
-		return (! gda_firebird_provider_run_sql (NULL, cnc, NULL, "DROP DATABASE"));
+	/* Make connection to database */
+	if (isc_attach_database (fcnc->status, strlen (name), (gchar *) name, &(db_handle),
+				 fcnc->dpb_length, fcnc->dpb_buffer)) {
+		gda_firebird_connection_make_error (cnc, 0);
+		return FALSE;
+	}
 	
-	return FALSE;
+	/* Drop database */
+	if (isc_drop_database (fcnc->status, &(db_handle))) {
+		gda_firebird_connection_make_error (cnc, 0);
+		return FALSE;
+	}
+	
+	return TRUE;
 }
 
 /* execute_command handler for the GdaFirebirdProvider class */
@@ -1043,22 +1138,48 @@ gda_firebird_provider_execute_command (GdaServerProvider *provider,
 {
 	isc_tr_handle *ftr;
 	GList *reclist = NULL;
+	gboolean commit_tr;
+	GdaTransaction *xaction;
 
 	g_return_val_if_fail (GDA_IS_FIREBIRD_PROVIDER (provider), NULL);
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
 	g_return_val_if_fail (cmd != NULL, NULL);
+	
+	commit_tr = FALSE;
 
-	ftr = gda_firebird_command_get_transaction (cmd);
+	/* Get transaction */
+	xaction = gda_command_get_transaction (cmd);
+	if (!GDA_IS_TRANSACTION (xaction))
+		xaction = gda_transaction_new ("local_tr");
+	
+	/* Get transaction handle */
+	ftr = g_object_get_data (G_OBJECT (xaction), TRANSACTION_DATA);
+	
+	/* Begin transaction if it was not already started  */
+	if (!ftr) {
 
-	/* parse command */
+		/* Start transaction */
+		commit_tr = gda_firebird_provider_begin_transaction (provider, cnc, xaction);
+	
+		/* Get transaction handle */
+		ftr = g_object_get_data (G_OBJECT (xaction), TRANSACTION_DATA);
+	}
+	
+	/* Parse command */
 	switch (gda_command_get_command_type (cmd)) {
-		case GDA_COMMAND_TYPE_SQL :
-	  		reclist = gda_firebird_provider_run_sql (reclist, cnc, ftr, gda_command_get_text (cmd));
+		case GDA_COMMAND_TYPE_SQL:
+			reclist = gda_firebird_provider_run_sql (reclist, cnc, ftr, gda_command_get_text (cmd));
 			break;
-		default: 
+		default:
 			break;
 	}
 
+	/* Commit transaction if it was not started directly by programmer */
+	if (commit_tr) {
+		gda_firebird_provider_commit_transaction (provider, cnc, xaction);
+		g_object_unref (xaction);
+	}
+	
 	return reclist;
 }
 
@@ -1083,7 +1204,7 @@ gda_firebird_provider_begin_transaction (GdaServerProvider *provider,
 
 	fcnc = g_object_get_data (G_OBJECT (cnc), CONNECTION_DATA);
 	if (!fcnc) {
-		gda_connection_add_error_string (cnc, _("Invalid Firebird handle"));
+		gda_connection_add_event_string (cnc, _("Invalid Firebird handle"));
 		return FALSE;
 	}
 
@@ -1117,13 +1238,13 @@ gda_firebird_provider_commit_transaction (GdaServerProvider *provider,
 
 	fcnc = g_object_get_data (G_OBJECT (cnc), CONNECTION_DATA);
 	if (!fcnc) {
-		gda_connection_add_error_string (cnc, _("Invalid Firebird handle"));
+		gda_connection_add_event_string (cnc, _("Invalid Firebird handle"));
 		return FALSE;
 	}
 
 	ftr = g_object_get_data (G_OBJECT (xaction), TRANSACTION_DATA);
 	if (!ftr) {
-		gda_connection_add_error_string (cnc, _("Invalid transaction handle"));
+		gda_connection_add_event_string (cnc, _("Invalid transaction handle"));
 		return FALSE;
 	}
 
@@ -1155,13 +1276,13 @@ gda_firebird_provider_rollback_transaction (GdaServerProvider *provider,
 
 	fcnc = g_object_get_data (G_OBJECT (cnc), CONNECTION_DATA);
 	if (!fcnc) {
-		gda_connection_add_error_string (cnc, _("Invalid Firebird handle"));
+		gda_connection_add_event_string (cnc, _("Invalid Firebird handle"));
 		return FALSE;
 	}
 
 	ftr = g_object_get_data (G_OBJECT (xaction), TRANSACTION_DATA);
 	if (!ftr) {
-		gda_connection_add_error_string (cnc, _("Invalid transaction handle"));
+		gda_connection_add_event_string (cnc, _("Invalid transaction handle"));
 		return FALSE;
 	}
 
@@ -1185,23 +1306,22 @@ gda_firebird_provider_supports (GdaServerProvider *provider,
 				GdaConnectionFeature feature)
 {
 	g_return_val_if_fail (GDA_IS_FIREBIRD_PROVIDER (provider), FALSE);
-	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
-                                                                                                                            
+
 	switch (feature) {
 		case GDA_CONNECTION_FEATURE_VIEWS:
 		case GDA_CONNECTION_FEATURE_SQL:
 		case GDA_CONNECTION_FEATURE_TRANSACTIONS:
+		case GDA_CONNECTION_FEATURE_AGGREGATES:
 			return TRUE;
 		case GDA_CONNECTION_FEATURE_TRIGGERS:
 		case GDA_CONNECTION_FEATURE_INDEXES:
 		case GDA_CONNECTION_FEATURE_PROCEDURES:
 		case GDA_CONNECTION_FEATURE_USERS:
 		case GDA_CONNECTION_FEATURE_BLOBS:
-		case GDA_CONNECTION_FEATURE_AGGREGATES:
-		default: 
+		default:
 			break;
 	}
-    
+
 	return FALSE;
 }
 
@@ -1224,14 +1344,16 @@ gda_firebird_provider_get_schema (GdaServerProvider *provider,
 			return fb_get_tables (cnc, params, TRUE);
 		case GDA_CONNECTION_SCHEMA_FIELDS:
 			return fb_get_fields_metadata (cnc, params);
+		case GDA_CONNECTION_SCHEMA_AGGREGATES:
+			return fb_get_aggregates (cnc, params);
 /*		case GDA_CONNECTION_SCHEMA_INDEXES:
 			return get_firebird_indexes (cnc, params);
 		case GDA_CONNECTION_SCHEMA_PROCEDURES:
 			return get_firebird_procedures (cnc, params);
-		case GDA_CONNECTION_SCHEMA_TRIGGERS:
-			return get_firebird_triggers (cnc, params);
 		case GDA_CONNECTION_SCHEMA_USERS:
-			return get_firebird_users (cnc, params);*/
+			return get_firebird_users (cnc, params);
+		case GDA_CONNECTION_SCHEMA_TRIGGERS:
+			return get_firebird_triggers (cnc, params);*/
 		default:
 			break;
 	}
@@ -1239,3 +1361,4 @@ gda_firebird_provider_get_schema (GdaServerProvider *provider,
 	return NULL;
 }
 
+
