@@ -60,13 +60,19 @@ static gboolean gda_mysql_provider_change_database (GdaServerProvider *provider,
 		                                    GdaConnection *cnc,
 		                                    const gchar *name);
 
-static gboolean gda_mysql_provider_create_database (GdaServerProvider *provider,
-						    GdaConnection *cnc,
-						    const gchar *name);
+static gchar *gda_mysql_provider_get_specs (GdaServerProvider *provider, GdaClientSpecsType type);
 
-static gboolean gda_mysql_provider_drop_database (GdaServerProvider *provider,
-						  GdaConnection *cnc,
-						  const gchar *name);
+static gboolean gda_mysql_provider_perform_action_params (GdaServerProvider *provider,
+							  GdaParameterList *params, GdaClientSpecsType type,
+							  GError **error);
+
+static gboolean gda_mysql_provider_create_database_cnc (GdaServerProvider *provider,
+						        GdaConnection *cnc,
+						        const gchar *name);
+
+static gboolean gda_mysql_provider_drop_database_cnc (GdaServerProvider *provider,
+						      GdaConnection *cnc,
+						      const gchar *name);
 
 static gboolean gda_mysql_provider_create_table (GdaServerProvider *provider,
                                                     GdaConnection *cnc,
@@ -144,8 +150,10 @@ gda_mysql_provider_class_init (GdaMysqlProviderClass *klass)
 	provider_class->get_server_version = gda_mysql_provider_get_server_version;
 	provider_class->get_database = gda_mysql_provider_get_database;
 	provider_class->change_database = gda_mysql_provider_change_database;
-	provider_class->create_database = gda_mysql_provider_create_database;
-	provider_class->drop_database = gda_mysql_provider_drop_database;
+	provider_class->get_specs = gda_mysql_provider_get_specs;
+	provider_class->perform_action_params = gda_mysql_provider_perform_action_params;
+	provider_class->create_database_cnc = gda_mysql_provider_create_database_cnc;
+	provider_class->drop_database_cnc = gda_mysql_provider_drop_database_cnc;
 	provider_class->create_table = gda_mysql_provider_create_table;
 	provider_class->drop_table = gda_mysql_provider_drop_table;
 	provider_class->create_index = gda_mysql_provider_create_index;
@@ -218,8 +226,71 @@ gda_mysql_provider_get_version (GdaServerProvider *provider)
 	return PACKAGE_VERSION;
 }
 
+/* generic function to open a MYSQL connection */
+static MYSQL *
+real_open_connection (const gchar *host, gint port, const gchar *socket,
+		      const gchar *db,
+		      const gchar *login, const gchar *password, 
+		      gboolean usessl, GError **error)
+{
+	MYSQL *mysql;
+	MYSQL *mysql_ret;
+	unsigned int mysqlflags = 0;
+#if MYSQL_VERSION_ID < 32200
+        gint err;
+#endif
+	
+	/* we can't have both a host/pair AND a unix_socket */
+        if ((host || (port >= 0)) && socket) {
+                g_set_error (error, 0, 0,
+			     _("You cannot provide a UNIX SOCKET if you also provide"
+			       " either a HOST or a PORT."));
+                return NULL;
+        }
+
+	/* provide the default of localhost:3306 if neither is provided */
+	if (!socket) {
+		if (!host)
+			host = "localhost";
+		else if (port <= 0)
+			port = 3306;
+	}
+
+	if (usessl)
+		 mysqlflags |= CLIENT_SSL;
+
+	mysql = g_new0 (MYSQL, 1);
+        mysql_init (mysql);
+        mysql_ret = mysql_real_connect (mysql, host, login, password,
+#if MYSQL_VERSION_ID >= 32200
+					db,
+#endif
+					port > 0? port : 0,
+					socket,
+					mysqlflags);
+	
+	
+	if (!mysql_ret) {
+		g_set_error (error, 0, 0, mysql_error (mysql));
+		g_free (mysql);
+		return NULL;
+	}
+	
+#if MYSQL_VERSION_ID < 32200
+	err = mysql_select_db (mysql, db);
+	if (err != 0) {
+		g_set_error (error, 0, 0, mysql_error (mysql))
+			mysql_close (mysql);
+		
+		return NULL;
+	}
+#endif
+	
+	return mysql;
+}
+
 /* open_connection handler for the GdaMysqlProvider class */
-static gboolean
+static 
 gda_mysql_provider_open_connection (GdaServerProvider *provider,
 				    GdaConnection *cnc,
 				    GdaQuarkList *params,
@@ -234,11 +305,10 @@ gda_mysql_provider_open_connection (GdaServerProvider *provider,
         const gchar *t_unix_socket = NULL;
         const gchar *t_use_ssl = NULL;
 	unsigned int mysqlflags = 0;
+
 	MYSQL *mysql;
-#if MYSQL_VERSION_ID < 32200
-        gint err;
-#endif
-	GdaError *error;
+	GdaConnectionEvent *error;
+	GError *gerror = NULL;
 	GdaMysqlProvider *myprv = (GdaMysqlProvider *) provider;
 
 	g_return_val_if_fail (GDA_IS_MYSQL_PROVIDER (myprv), FALSE);
@@ -258,51 +328,19 @@ gda_mysql_provider_open_connection (GdaServerProvider *provider,
 	if (password && *password != '\0')
 		t_password = password;
 
-	if (t_use_ssl && atoi (t_use_ssl) == 1)
-		mysqlflags |= CLIENT_SSL;
-
-	/* we can't have both a host/pair AND a unix_socket */
-	if ((t_host || t_port) && t_unix_socket) {
-		gda_connection_add_error_string (
-			cnc, _("You cannot provide a UNIX_SOCKET if you also provide"
-			       " either a HOST or a PORT."));
-		return FALSE;
-	}
-
-	/* provide the default of localhost:3306 if neither is provided */
-	if (!t_unix_socket) {
-		if (!t_host)
-			t_host = "localhost";
-		else if (!t_port)
-			t_port = "3306";
-	}
-	
-	mysql = g_new0 (MYSQL, 1);
-	mysql_init (mysql);
-	mysql = mysql_real_connect (mysql, t_host, t_user, t_password,
-#if MYSQL_VERSION_ID >= 32200
-				    t_db,
-#endif
-				    t_port ? atoi (t_port) : 0,
-				    t_unix_socket,
-				    mysqlflags);
+	mysql = real_open_connection (t_host, t_port ? atoi (t_port) : 0, t_unix_socket,
+				      t_db, t_user, t_password, t_use_ssl ? TRUE : FALSE, &gerror);
 	if (!mysql) {
-		error = gda_mysql_make_error (mysql);
-		gda_connection_add_error (cnc, error);
+		error = gda_connection_event_new (GDA_CONNECTION_EVENT_ERROR);
+		gda_connection_event_set_description (error, gerror && gerror->message ? 
+					   gerror->message : "NO DESCRIPTION");
+                gda_connection_event_set_code (error, gerror ? gerror->code : -1);
+		if (gerror)
+			g_error_free (gerror);
+		gda_connection_add_event (cnc, error);
 
 		return FALSE;
 	}
-#if MYSQL_VERSION_ID < 32200
-	err = mysql_select_db (mysql, t_db);
-	if (err != 0) {
-		error = gda_mysql_make_error (mysql);
-		mysql_close (mysql);
-		gda_connection_add_error (cnc, error);
-
-		return FALSE;
-	}
-#endif
-
 	g_object_set_data (G_OBJECT (cnc), OBJECT_DATA_MYSQL_HANDLE, mysql);
 
 	return TRUE;
@@ -340,7 +378,7 @@ gda_mysql_provider_get_server_version (GdaServerProvider *provider, GdaConnectio
 	
 	mysql = g_object_get_data (G_OBJECT (cnc), OBJECT_DATA_MYSQL_HANDLE);
 	if (!mysql) {
-		gda_connection_add_error_string (cnc, _("Invalid MySQL handle"));
+		gda_connection_add_event_string (cnc, _("Invalid MySQL handle"));
 		return FALSE;
 	}
 
@@ -356,7 +394,7 @@ process_sql_commands (GList *reclist, GdaConnection *cnc, const gchar *sql)
 
 	mysql = g_object_get_data (G_OBJECT (cnc), OBJECT_DATA_MYSQL_HANDLE);
 	if (!mysql) {
-		gda_connection_add_error_string (cnc, _("Invalid MYSQL handle"));
+		gda_connection_add_event_string (cnc, _("Invalid MYSQL handle"));
 		return NULL;
 	}
 
@@ -371,6 +409,7 @@ process_sql_commands (GList *reclist, GdaConnection *cnc, const gchar *sql)
 			gint rc;
 			MYSQL_RES *mysql_res;
 			GdaMysqlRecordset *recset;
+			gchar *tststr;
 
 			/* if the connection is in read-only mode, just allow SELECT,
 			   SHOW commands */
@@ -381,7 +420,7 @@ process_sql_commands (GList *reclist, GdaConnection *cnc, const gchar *sql)
 				s = g_strstrip (g_strdup (arr[n]));
 				if (g_ascii_strncasecmp (s, "select", strlen ("select")) &&
 				    g_ascii_strncasecmp (s, "show", strlen ("show"))) {
-					gda_connection_add_error_string (
+					gda_connection_add_event_string (
 						cnc, "Command '%s' cannot be executed in read-only mode", arr[n]);
 					break;
 				}
@@ -392,16 +431,55 @@ process_sql_commands (GList *reclist, GdaConnection *cnc, const gchar *sql)
 			/* execute the command on the server */
 			rc = mysql_real_query (mysql, arr[n], strlen (arr[n]));
 			if (rc != 0) {
-				gda_connection_add_error (cnc, gda_mysql_make_error (mysql));
+				gda_connection_add_event (cnc, gda_mysql_make_error (mysql));
 				break;
 			}
 
-			mysql_res = mysql_store_result (mysql);
-			recset = gda_mysql_recordset_new (cnc, mysql_res, mysql);
-			if (GDA_IS_MYSQL_RECORDSET (recset)) {
-				gda_data_model_set_command_text ( (GdaDataModel*) recset, arr[n]);
-				gda_data_model_set_command_type ( (GdaDataModel*) recset, GDA_COMMAND_TYPE_SQL);
-				reclist = g_list_append (reclist, recset);
+			/* command was executed OK */
+			g_strchug (arr[n]);
+			tststr = arr[n];
+			if (! g_ascii_strncasecmp (tststr, "SELECT", 6) ||
+			    ! g_ascii_strncasecmp (tststr, "SHOW", 4) ||
+			    ! g_ascii_strncasecmp (tststr, "DESCRIBE", 6) ||
+			    ! g_ascii_strncasecmp (tststr, "EXPLAIN", 7)) {
+				mysql_res = mysql_store_result (mysql);
+				recset = gda_mysql_recordset_new (cnc, mysql_res, mysql);
+				if (GDA_IS_MYSQL_RECORDSET (recset)) {
+					gda_data_model_set_command_text ( (GdaDataModel*) recset, arr[n]);
+					gda_data_model_set_command_type ( (GdaDataModel*) recset, GDA_COMMAND_TYPE_SQL);
+					reclist = g_list_append (reclist, recset);
+				}
+			}
+			else {
+				int changes;
+				GdaConnectionEvent *event;
+				gchar *str, *tmp, *ptr;
+				
+				/* don't return a data model */
+				reclist = g_list_append (reclist, NULL);
+				
+				/* generate a notice about changes */
+				changes = mysql_affected_rows (mysql);
+				event = gda_connection_event_new (GDA_CONNECTION_EVENT_NOTICE);
+				ptr = tststr;
+				while (*ptr && (*ptr != ' ') && (*ptr != '\t') &&
+				       (*ptr != '\n'))
+					ptr++;
+				*ptr = 0;
+				tmp = g_ascii_strup (tststr, -1);
+				if (!strcmp (tmp, "INSERT")) {
+					if (mysql_insert_id (mysql) != 0)
+						str = g_strdup_printf ("%s %lld %d", tmp, 
+								       mysql_insert_id (mysql),
+								       changes);
+					else
+						str = g_strdup_printf ("%s NOID %d", tmp, changes);
+				}
+				else
+					str = g_strdup_printf ("%s %d", tmp, changes);
+				gda_connection_event_set_description (event, str);
+				g_free (str);
+				gda_connection_add_event (cnc, event);
 			}
 
 			n++;
@@ -426,7 +504,7 @@ gda_mysql_provider_get_database (GdaServerProvider *provider,
 
 	mysql = g_object_get_data (G_OBJECT (cnc), OBJECT_DATA_MYSQL_HANDLE);
 	if (!mysql) {
-		gda_connection_add_error_string (cnc, _("Invalid MYSQL handle"));
+		gda_connection_add_event_string (cnc, _("Invalid MYSQL handle"));
 		return NULL;
 	}
 
@@ -449,13 +527,13 @@ gda_mysql_provider_change_database (GdaServerProvider *provider,
 
 	mysql = g_object_get_data (G_OBJECT (cnc), OBJECT_DATA_MYSQL_HANDLE);
 	if (!mysql) {
-		gda_connection_add_error_string (cnc, _("Invalid MYSQL handle"));
+		gda_connection_add_event_string (cnc, _("Invalid MYSQL handle"));
 		return FALSE;
 	}
 
 	rc = mysql_select_db (mysql, name);
 	if (rc != 0) {
-		gda_connection_add_error (cnc, gda_mysql_make_error (mysql));
+		gda_connection_add_event (cnc, gda_mysql_make_error (mysql));
 		return FALSE;
 	}
 
@@ -463,11 +541,141 @@ gda_mysql_provider_change_database (GdaServerProvider *provider,
 
 }
 
+/* get_specs_create_database handler for the GdaMysqlProvider class */
+static gchar *
+gda_mysql_provider_get_specs (GdaServerProvider *provider, GdaClientSpecsType type)
+{
+	gchar *specs, *file;
+        gint len;
+	GdaMysqlProvider *myprv = (GdaMysqlProvider *) provider;
+
+	g_return_val_if_fail (GDA_IS_MYSQL_PROVIDER (myprv), FALSE);
+
+	switch (type) {
+	case GDA_CLIENT_SPECS_CREATE_DATABASE:
+		file = g_build_filename (LIBGDA_DATA_DIR, "mysql_specs_create_db.xml", NULL);
+		if (g_file_get_contents (file, &specs, &len, NULL))
+			return specs;
+		else
+			return NULL;
+		break;
+	default: 
+		return NULL;
+	}
+}
+
+#define string_from_string_param(param) \
+	(param && gda_parameter_get_value (param) && !gda_value_is_null (gda_parameter_get_value (param))) ? \
+	gda_value_get_string (gda_parameter_get_value (param)) : NULL
+
+#define int_from_int_param(param) \
+	(param && gda_parameter_get_value (param) && !gda_value_is_null (gda_parameter_get_value (param))) ? \
+	gda_value_get_integer (gda_parameter_get_value (param)) : -1
+
+#define bool_from_bool_param(param) \
+	(param && gda_parameter_get_value (param) && !gda_value_is_null (gda_parameter_get_value (param))) ? \
+	gda_value_get_boolean (gda_parameter_get_value (param)) : FALSE
+
 /* create_database handler for the GdaMysqlProvider class */
 static gboolean
-gda_mysql_provider_create_database (GdaServerProvider *provider,
-				    GdaConnection *cnc,
-				    const gchar *name)
+gda_mysql_provider_perform_action_params (GdaServerProvider *provider,
+					  GdaParameterList *params, 
+					  GdaClientSpecsType type, GError **error)
+{
+	GdaMysqlProvider *myprv = (GdaMysqlProvider *) provider;
+	MYSQL *mysql;
+	int res;
+	gboolean retval = TRUE;
+	GdaParameter *param = NULL;
+	GString *string;
+	
+	const gchar *login = NULL;
+	const gchar *password = NULL;
+	const gchar *host = NULL;
+	gint         port = -1;
+	const gchar *socket = NULL;
+	gboolean     usessl = FALSE;
+	const gchar *dbname = NULL;
+	const gchar *encoding = NULL;
+	const gchar *collate = NULL;
+
+
+	g_return_val_if_fail (GDA_IS_MYSQL_PROVIDER (myprv), FALSE);
+
+	switch (type) {
+	case GDA_CLIENT_SPECS_CREATE_DATABASE:	
+		if (params) {
+			param = gda_parameter_list_find (params, "HOST");
+			host = string_from_string_param (param);
+			
+			param = gda_parameter_list_find (params, "PORT");
+			port = int_from_int_param (param);
+			
+			param = gda_parameter_list_find (params, "UNIX_SOCKET");
+			socket = string_from_string_param (param);
+			
+			param = gda_parameter_list_find (params, "USE_SSL");
+			usessl = bool_from_bool_param (param);
+			
+			param = gda_parameter_list_find (params, "DATABASE");
+			dbname = string_from_string_param (param);
+			
+			param = gda_parameter_list_find (params, "ENCODING");
+			encoding = string_from_string_param (param);
+			
+			param = gda_parameter_list_find (params, "COLLATE");
+			collate = string_from_string_param (param);
+			
+			param = gda_parameter_list_find (params, "ADM_LOGIN");
+			login = string_from_string_param (param);
+			
+			param = gda_parameter_list_find (params, "ADM_PASSWORD");
+			password = string_from_string_param (param);
+		}
+		if (!dbname) {
+			g_set_error (error, 0, 0,
+				     _("Missing parameter 'DATABASE'"));
+			return FALSE;
+		}
+		
+		mysql = real_open_connection (host, port, socket,
+					      "mysql", login, password, usessl, error);
+		if (!mysql)
+			return FALSE;
+		
+		/* Ask to create a database */
+		string = g_string_new ("CREATE DATABASE ");
+		g_string_append (string, dbname);
+		if (encoding)
+			g_string_append_printf (string, " CHARACTER SET %s", encoding);
+		if (collate)
+			g_string_append_printf (string, " COLLATE %s", collate);
+		
+		res = mysql_query (mysql, string->str);
+		g_string_free (string, TRUE);
+		
+		if (res) {
+			g_set_error (error, 0, 0, mysql_error (mysql));
+			retval = FALSE;
+		}
+		
+		mysql_close (mysql);
+		break;
+	default:
+		g_set_error (error, 0, 0,
+			     _("Method not handled by this provider"));
+		return FALSE;
+	}
+
+	return retval;
+}
+
+
+/* create_database_cnc handler for the GdaMysqlProvider class */
+static gboolean
+gda_mysql_provider_create_database_cnc (GdaServerProvider *provider,
+				        GdaConnection *cnc,
+				        const gchar *name)
 {
 	gint rc;
 	gchar *sql;
@@ -480,7 +688,7 @@ gda_mysql_provider_create_database (GdaServerProvider *provider,
 
 	mysql = g_object_get_data (G_OBJECT (cnc), OBJECT_DATA_MYSQL_HANDLE);
 	if (!mysql) {
-		gda_connection_add_error_string (cnc, _("Invalid MYSQL handle"));
+		gda_connection_add_event_string (cnc, _("Invalid MYSQL handle"));
 		return FALSE;
 	}
 
@@ -488,7 +696,7 @@ gda_mysql_provider_create_database (GdaServerProvider *provider,
 	rc = mysql_query (mysql, sql);
 	g_free (sql);
 	if (rc != 0) {
-		gda_connection_add_error (cnc, gda_mysql_make_error (mysql));
+		gda_connection_add_event (cnc, gda_mysql_make_error (mysql));
 		return FALSE;
 	}
 
@@ -497,9 +705,9 @@ gda_mysql_provider_create_database (GdaServerProvider *provider,
 
 /* drop_database handler for the GdaMysqlProvider class */
 static gboolean
-gda_mysql_provider_drop_database (GdaServerProvider *provider,
-				  GdaConnection *cnc,
-				  const gchar *name)
+gda_mysql_provider_drop_database_cnc (GdaServerProvider *provider,
+				      GdaConnection *cnc,
+				      const gchar *name)
 {
 	gint rc;
 	gchar *sql;
@@ -512,7 +720,7 @@ gda_mysql_provider_drop_database (GdaServerProvider *provider,
 
 	mysql = g_object_get_data (G_OBJECT (cnc), OBJECT_DATA_MYSQL_HANDLE);
 	if (!mysql) {
-		gda_connection_add_error_string (cnc, _("Invalid MYSQL handle"));
+		gda_connection_add_event_string (cnc, _("Invalid MYSQL handle"));
 		return FALSE;
 	}
 
@@ -520,7 +728,7 @@ gda_mysql_provider_drop_database (GdaServerProvider *provider,
 	rc = mysql_query (mysql, sql);
 	g_free (sql);
 	if (rc != 0) {
-		gda_connection_add_error (cnc, gda_mysql_make_error (mysql));
+		gda_connection_add_event (cnc, gda_mysql_make_error (mysql));
 		return FALSE;
 	}
 
@@ -554,7 +762,7 @@ gda_mysql_provider_create_table (GdaServerProvider *provider,
 	/* check for valid MySQL handle */
 	mysql = g_object_get_data (G_OBJECT (cnc), OBJECT_DATA_MYSQL_HANDLE);
 	if (!mysql) {
-		gda_connection_add_error_string (cnc, _("Invalid MYSQL handle"));
+		gda_connection_add_event_string (cnc, _("Invalid MYSQL handle"));
 		return FALSE;
 	}
 
@@ -634,7 +842,7 @@ gda_mysql_provider_create_table (GdaServerProvider *provider,
 
 		/* auto increment */
 		if (gda_column_get_auto_increment (dmca) == TRUE)
-			g_string_append_printf (sql, "AUTO_INCREMENT");
+			g_string_append_printf (sql, " AUTO_INCREMENT");
 
 		/* (primary) key */
 		if (gda_column_get_primary_key (dmca) == TRUE)
@@ -664,7 +872,7 @@ gda_mysql_provider_create_table (GdaServerProvider *provider,
 	/* execute sql command */
 	rc = mysql_query (mysql, sql->str);
 	if (rc != 0) {
-		gda_connection_add_error (cnc, gda_mysql_make_error (mysql));
+		gda_connection_add_event (cnc, gda_mysql_make_error (mysql));
 		return FALSE;
 	}
 
@@ -707,7 +915,7 @@ gda_mysql_provider_drop_table (GdaServerProvider *provider,
 
 	mysql = g_object_get_data (G_OBJECT (cnc), OBJECT_DATA_MYSQL_HANDLE);
 	if (!mysql) {
-		gda_connection_add_error_string (cnc, _("Invalid MYSQL handle"));
+		gda_connection_add_event_string (cnc, _("Invalid MYSQL handle"));
 		return FALSE;
 	}
 
@@ -715,7 +923,7 @@ gda_mysql_provider_drop_table (GdaServerProvider *provider,
 	rc = mysql_query (mysql, sql);
 	g_free (sql);
 	if (rc != 0) {
-		gda_connection_add_error (cnc, gda_mysql_make_error (mysql));
+		gda_connection_add_event (cnc, gda_mysql_make_error (mysql));
 		return FALSE;
 	}
 
@@ -747,7 +955,7 @@ gda_mysql_provider_create_index (GdaServerProvider *provider,
 	/* get MySQL handle */
 	mysql = g_object_get_data (G_OBJECT (cnc), OBJECT_DATA_MYSQL_HANDLE);
 	if (!mysql) {
-		gda_connection_add_error_string (cnc, _("Invalid MYSQL handle"));
+		gda_connection_add_event_string (cnc, _("Invalid MYSQL handle"));
 		return FALSE;
 	}
 
@@ -813,7 +1021,7 @@ gda_mysql_provider_create_index (GdaServerProvider *provider,
         /* execute sql command */
 	rc = mysql_query (mysql, sql->str);
 	if (rc != 0) {
-		gda_connection_add_error (cnc, gda_mysql_make_error (mysql));
+		gda_connection_add_event (cnc, gda_mysql_make_error (mysql));
 		return FALSE;
 	}
 
@@ -844,7 +1052,7 @@ gda_mysql_provider_drop_index (GdaServerProvider *provider,
 	/* get MySQL handle */
 	mysql = g_object_get_data (G_OBJECT (cnc), OBJECT_DATA_MYSQL_HANDLE);
 	if (!mysql) {
-		gda_connection_add_error_string (cnc, _("Invalid MYSQL handle"));
+		gda_connection_add_event_string (cnc, _("Invalid MYSQL handle"));
 		return FALSE;
 	}
 
@@ -856,7 +1064,7 @@ gda_mysql_provider_drop_index (GdaServerProvider *provider,
 	rc = mysql_query (mysql, sql);
 	g_free (sql);
 	if (rc != 0) {
-		gda_connection_add_error (cnc, gda_mysql_make_error (mysql));
+		gda_connection_add_event (cnc, gda_mysql_make_error (mysql));
 		return FALSE;
 	}
 
@@ -914,7 +1122,7 @@ gda_mysql_provider_get_last_insert_id (GdaServerProvider *provider,
 
 	mysql = g_object_get_data (G_OBJECT (cnc), OBJECT_DATA_MYSQL_HANDLE);
 	if (!mysql) {
-		gda_connection_add_error_string (cnc, _("Invalid MYSQL handle"));
+		gda_connection_add_event_string (cnc, _("Invalid MYSQL handle"));
 		return NULL;
 	}
 
@@ -937,12 +1145,12 @@ gda_mysql_provider_begin_transaction (GdaServerProvider *provider,
 
 	mysql = g_object_get_data (G_OBJECT (cnc), OBJECT_DATA_MYSQL_HANDLE);
 	if (!mysql) {
-		gda_connection_add_error_string (cnc, _("Invalid MYSQL handle"));
+		gda_connection_add_event_string (cnc, _("Invalid MYSQL handle"));
 		return FALSE;
 	}
 
 	if (gda_connection_get_options (cnc) & GDA_CONNECTION_OPTIONS_READ_ONLY) {
-		gda_connection_add_error_string (cnc, _("Transactions are not supported in read-only mode"));
+		gda_connection_add_event_string (cnc, _("Transactions are not supported in read-only mode"));
 		return FALSE;
 	}
 
@@ -965,14 +1173,14 @@ gda_mysql_provider_begin_transaction (GdaServerProvider *provider,
 	}
 
 	if (rc != 0) {
-		gda_connection_add_error (cnc, gda_mysql_make_error (mysql));
+		gda_connection_add_event (cnc, gda_mysql_make_error (mysql));
 		return FALSE;
 	}
 
 	/* start the transaction */
 	rc = mysql_real_query (mysql, "BEGIN", strlen ("BEGIN"));
 	if (rc != 0) {
-		gda_connection_add_error (cnc, gda_mysql_make_error (mysql));
+		gda_connection_add_event (cnc, gda_mysql_make_error (mysql));
 		return FALSE;
 	}
 
@@ -994,18 +1202,18 @@ gda_mysql_provider_commit_transaction (GdaServerProvider *provider,
 
 	mysql = g_object_get_data (G_OBJECT (cnc), OBJECT_DATA_MYSQL_HANDLE);
 	if (!mysql) {
-		gda_connection_add_error_string (cnc, _("Invalid MYSQL handle"));
+		gda_connection_add_event_string (cnc, _("Invalid MYSQL handle"));
 		return FALSE;
 	}
 
 	if (gda_connection_get_options (cnc) & GDA_CONNECTION_OPTIONS_READ_ONLY) {
-		gda_connection_add_error_string (cnc, _("Transactions are not supported in read-only mode"));
+		gda_connection_add_event_string (cnc, _("Transactions are not supported in read-only mode"));
 		return FALSE;
 	}
 
 	rc = mysql_real_query (mysql, "COMMIT", strlen ("COMMIT"));
 	if (rc != 0) {
-		gda_connection_add_error (cnc, gda_mysql_make_error (mysql));
+		gda_connection_add_event (cnc, gda_mysql_make_error (mysql));
 		return FALSE;
 	}
 
@@ -1027,18 +1235,18 @@ gda_mysql_provider_rollback_transaction (GdaServerProvider *provider,
 
 	mysql = g_object_get_data (G_OBJECT (cnc), OBJECT_DATA_MYSQL_HANDLE);
 	if (!mysql) {
-		gda_connection_add_error_string (cnc, _("Invalid MYSQL handle"));
+		gda_connection_add_event_string (cnc, _("Invalid MYSQL handle"));
 		return FALSE;
 	}
 
 	if (gda_connection_get_options (cnc) & GDA_CONNECTION_OPTIONS_READ_ONLY) {
-		gda_connection_add_error_string (cnc, _("Transactions are not supported in read-only mode"));
+		gda_connection_add_event_string (cnc, _("Transactions are not supported in read-only mode"));
 		return FALSE;
 	}
 
 	rc = mysql_real_query (mysql, "ROLLBACK", strlen ("ROLLBACK"));
 	if (rc != 0) {
-		gda_connection_add_error (cnc, gda_mysql_make_error (mysql));
+		gda_connection_add_event (cnc, gda_mysql_make_error (mysql));
 		return FALSE;
 	}
 
@@ -1487,7 +1695,7 @@ get_mysql_types (GdaConnection *cnc, GdaParameterList *params)
 		value_list = g_list_append (value_list, gda_value_new_string (types[i].name));
 		value_list = g_list_append (value_list, gda_value_new_string (types[i].owner));
 		value_list = g_list_append (value_list, gda_value_new_string (types[i].comments));
-		value_list = g_list_append (value_list, gda_value_new_type (types[i].type));
+		value_list = g_list_append (value_list, gda_value_new_gdatype (types[i].type));
 		value_list = g_list_append (value_list, gda_value_new_string (types[i].synonyms));
 
 		gda_data_model_append_values (GDA_DATA_MODEL (recset), value_list);
@@ -1551,11 +1759,17 @@ field_row_to_value_list (MYSQL_ROW mysql_row)
 	value_list = g_list_append (value_list, gda_value_new_boolean (FALSE));
 
 	/* references */
-	value_list = g_list_append (value_list, gda_value_new_string (mysql_row[5]));
+	value_list = g_list_append (value_list, gda_value_new_string (""));
 
 	/* default value */
 	value_list = g_list_append (value_list, gda_value_new_string (mysql_row[4]));
 
+	/* Extra column */
+	if (!strcmp (mysql_row[5], "auto_increment"))
+		value_list = g_list_append (value_list, gda_value_new_string ("AUTO_INCREMENT"));
+	else
+		value_list = g_list_append (value_list, gda_value_new_string (""));
+
 	return value_list;
 }
 
@@ -1582,7 +1796,8 @@ get_table_fields (GdaConnection *cnc, GdaParameterList *params)
 		{ N_("Primary key?")	, GDA_VALUE_TYPE_BOOLEAN },
 		{ N_("Unique index?")	, GDA_VALUE_TYPE_BOOLEAN },
 		{ N_("References")	, GDA_VALUE_TYPE_STRING  },
-		{ N_("Default value")   , GDA_VALUE_TYPE_STRING  }
+		{ N_("Default value")   , GDA_VALUE_TYPE_STRING  },
+		{ N_("Extra attributes"), GDA_VALUE_TYPE_STRING  }
 	};
 
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
@@ -1590,14 +1805,14 @@ get_table_fields (GdaConnection *cnc, GdaParameterList *params)
 
 	mysql = g_object_get_data (G_OBJECT (cnc), OBJECT_DATA_MYSQL_HANDLE);
 	if (!mysql) {
-		gda_connection_add_error_string (cnc, _("Invalid MySQL handle"));
+		gda_connection_add_event_string (cnc, _("Invalid MySQL handle"));
 		return NULL;
 	}
 
 	/* get parameters sent by client */
 	par = gda_parameter_list_find (params, "name");
 	if (!par) {
-		gda_connection_add_error_string (
+		gda_connection_add_event_string (
 			cnc,
 			_("Table name is needed but none specified in parameter list"));
 		return NULL;
@@ -1605,7 +1820,7 @@ get_table_fields (GdaConnection *cnc, GdaParameterList *params)
 
 	table_name = gda_value_get_string ((GdaValue *) gda_parameter_get_value (par));
 	if (!table_name) {
-		gda_connection_add_error_string (
+		gda_connection_add_event_string (
 			cnc,
 			_("Table name is needed but none specified in parameter list"));
 		return NULL;
@@ -1616,7 +1831,7 @@ get_table_fields (GdaConnection *cnc, GdaParameterList *params)
 	rc = mysql_real_query (mysql, cmd_str, strlen (cmd_str));
 	g_free (cmd_str);
 	if (rc != 0) {
-		gda_connection_add_error (cnc, gda_mysql_make_error (mysql));
+		gda_connection_add_event (cnc, gda_mysql_make_error (mysql));
 		return NULL;
 	}
 
@@ -1624,7 +1839,7 @@ get_table_fields (GdaConnection *cnc, GdaParameterList *params)
 	rows = mysql_num_rows (mysql_res);
 
 	/* fill in the recordset to be returned */
-	recset = (GdaDataModelArray *) gda_data_model_array_new (9);
+	recset = (GdaDataModelArray *) gda_data_model_array_new (sizeof fields_desc / sizeof fields_desc[0]);
 	for (r = 0; r < sizeof (fields_desc) / sizeof (fields_desc[0]); r++) {
 /*
 		gint defined_size =  (fields_desc[r].type == GDA_VALUE_TYPE_STRING) ? 64 : 
@@ -1662,7 +1877,7 @@ get_table_fields (GdaConnection *cnc, GdaParameterList *params)
 		}
 
 		gda_data_model_append_values (GDA_DATA_MODEL (recset),
-					   (const GList *) value_list);
+					      (const GList *) value_list);
 
 		g_list_foreach (value_list, (GFunc) gda_value_free, NULL);
 		g_list_free (value_list);
@@ -1720,7 +1935,7 @@ gda_mysql_provider_value_to_sql_string (
 	if (!val_str)
 		return NULL;
 
-	switch (from->type) {
+	switch (GDA_VALUE_TYPE(from)) {
 		case GDA_VALUE_TYPE_BIGINT :
 		case GDA_VALUE_TYPE_DOUBLE :
 		case GDA_VALUE_TYPE_INTEGER :
@@ -1742,7 +1957,7 @@ gda_mysql_provider_value_to_sql_string (
 			mysql = g_object_get_data (G_OBJECT (cnc), OBJECT_DATA_MYSQL_HANDLE);
 			
 			if (!mysql) {
-				gda_connection_add_error_string (cnc, _("Invalid MYSQL handle"));
+				gda_connection_add_event_string (cnc, _("Invalid MYSQL handle"));
 				return NULL;
 			}
 			quoted = ret = g_malloc(strlen(val_str) * 2 + 3);
