@@ -1,9 +1,10 @@
 /* GDA Oracle provider
- * Copyright (C) 2000-2002 The GNOME Foundation.
+ * Copyright (C) 2000 - 2005 The GNOME Foundation.
  *
  * AUTHORS:
  *	Rodrigo Moya <rodrigo@gnome-db.org>
  *	Tim Coleman <tim@timcoleman.com>
+ *      Vivien Malerba <malerba@gnome-db.org>
  *
  * This Library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public License as
@@ -53,9 +54,9 @@ static const gchar *gda_oracle_provider_get_database (GdaServerProvider *provide
 static gboolean gda_oracle_provider_create_database (GdaServerProvider *provider,
 						    GdaConnection *cnc,
 						    const gchar *name);
-static gboolean gda_oracle_provider_drop_database (GdaServerProvider *provider,
-						  GdaConnection *cnc,
-						  const gchar *name);
+static gboolean gda_oracle_provider_drop_database_cnc (GdaServerProvider *provider,
+						       GdaConnection *cnc,
+						       const gchar *name);
 static GList *gda_oracle_provider_execute_command (GdaServerProvider *provider,
 						  GdaConnection *cnc,
 						  GdaCommand *cmd,
@@ -108,8 +109,8 @@ gda_oracle_provider_class_init (GdaOracleProviderClass *klass)
 	provider_class->close_connection = gda_oracle_provider_close_connection;
 	provider_class->get_server_version = gda_oracle_provider_get_server_version;
 	provider_class->get_database = gda_oracle_provider_get_database;
-	provider_class->create_database = gda_oracle_provider_create_database;
-	provider_class->drop_database = gda_oracle_provider_drop_database;
+	provider_class->create_database_cnc = gda_oracle_provider_create_database;
+	provider_class->drop_database_cnc = gda_oracle_provider_drop_database_cnc;
 	provider_class->execute_command = gda_oracle_provider_execute_command;
 	provider_class->begin_transaction = gda_oracle_provider_begin_transaction;
 	provider_class->commit_transaction = gda_oracle_provider_commit_transaction;
@@ -192,7 +193,7 @@ gda_oracle_provider_open_connection (GdaServerProvider *provider,
 	/* check we have a TNS name to connect to */
 
 	if ((tnsname = gda_quark_list_find (params, "TNSNAME")) == NULL) {
-		gda_connection_add_error_string (cnc, 
+		gda_connection_add_event_string (cnc, 
 		        _("No TNS name supplied"));
 		return FALSE;
 	}
@@ -208,7 +209,7 @@ gda_oracle_provider_open_connection (GdaServerProvider *provider,
 				(void (*)(dvoid *, dvoid *)) 0);
 
 	if (result != OCI_SUCCESS) {
-		gda_connection_add_error_string (cnc, 
+		gda_connection_add_event_string (cnc, 
 			_("Could not initialize Oracle"));
 		return FALSE;
 	}
@@ -219,7 +220,7 @@ gda_oracle_provider_open_connection (GdaServerProvider *provider,
 				(size_t) 0, 
 				(dvoid **) 0);
 	if (result != OCI_SUCCESS) {
-		gda_connection_add_error_string (cnc, 
+		gda_connection_add_event_string (cnc, 
 			_("Could not initialize the Oracle environment"));
 		return FALSE;
 	}
@@ -419,7 +420,7 @@ gda_oracle_provider_close_connection (GdaServerProvider *provider, GdaConnection
 				priv_data->herr,
 				priv_data->hsession,
 				OCI_DEFAULT))) {
-		gda_connection_add_error (cnc, 
+		gda_connection_add_event (cnc, 
 			gda_oracle_make_error (priv_data->herr, OCI_HTYPE_ERROR, __FILE__, __LINE__));
 		return FALSE;
 	}
@@ -463,7 +464,7 @@ gda_oracle_provider_get_server_version (GdaServerProvider *provider,
 	/* Get the OracleConnectionData */
 	priv_data = g_object_get_data (G_OBJECT (cnc), OBJECT_DATA_ORACLE_HANDLE);
 	if (!priv_data) {
-		gda_connection_add_error_string (cnc, _("Invalid Oracle handle"));
+		gda_connection_add_event_string (cnc, _("Invalid Oracle handle"));
 		return NULL;
 	}
 
@@ -495,7 +496,7 @@ prepare_oracle_statement (GdaConnection *cnc, GdaParameterList *params, gchar *s
 	/* Get the OracleConnectionData */
 	priv_data = g_object_get_data (G_OBJECT (cnc), OBJECT_DATA_ORACLE_HANDLE);
 	if (!priv_data) {
-		gda_connection_add_error_string (cnc, _("Invalid Oracle handle"));
+		gda_connection_add_event_string (cnc, _("Invalid Oracle handle"));
 		return NULL;
 	}
 
@@ -570,7 +571,7 @@ process_sql_commands (GList *reclist, GdaConnection *cnc,
 
 	priv_data = g_object_get_data (G_OBJECT (cnc), OBJECT_DATA_ORACLE_HANDLE);
 	if (!priv_data) {
-		gda_connection_add_error_string (cnc, _("Invalid Oracle handle"));
+		gda_connection_add_event_string (cnc, _("Invalid Oracle handle"));
 		return NULL;
 	}
 
@@ -719,7 +720,7 @@ gda_oracle_provider_get_database (GdaServerProvider *provider,
 
 	priv_data = g_object_get_data (G_OBJECT (cnc), OBJECT_DATA_ORACLE_HANDLE);
 	if (!priv_data) {
-		gda_connection_add_error_string (cnc, _("Invalid Oracle handle"));
+		gda_connection_add_event_string (cnc, _("Invalid Oracle handle"));
 		return FALSE;
 	}
 
@@ -742,7 +743,7 @@ gda_oracle_provider_create_database (GdaServerProvider *provider,
 
 	priv_data = g_object_get_data (G_OBJECT (cnc), OBJECT_DATA_ORACLE_HANDLE);
 	if (!priv_data) {
-		gda_connection_add_error_string (cnc, _("Invalid Oracle handle"));
+		gda_connection_add_event_string (cnc, _("Invalid Oracle handle"));
 		return FALSE;
 	}
 
@@ -750,11 +751,11 @@ gda_oracle_provider_create_database (GdaServerProvider *provider,
 	return FALSE;
 }
 
-/* drop_database handler for the GdaOracleProvider class */
+/* drop_database_cnc handler for the GdaOracleProvider class */
 static gboolean
-gda_oracle_provider_drop_database (GdaServerProvider *provider,
-				  GdaConnection *cnc,
-				  const gchar *name)
+gda_oracle_provider_drop_database_cnc (GdaServerProvider *provider,
+				       GdaConnection *cnc,
+				       const gchar *name)
 {
 	GdaOracleConnectionData *priv_data;
 	GdaOracleProvider *ora_prv = (GdaOracleProvider *) provider;
@@ -765,7 +766,7 @@ gda_oracle_provider_drop_database (GdaServerProvider *provider,
 
 	priv_data = g_object_get_data (G_OBJECT (cnc), OBJECT_DATA_ORACLE_HANDLE);
 	if (!priv_data) {
-		gda_connection_add_error_string (cnc, _("Invalid Oracle handle"));
+		gda_connection_add_event_string (cnc, _("Invalid Oracle handle"));
 		return FALSE;
 	}
 
@@ -795,7 +796,7 @@ gda_oracle_provider_execute_command (GdaServerProvider *provider,
 
 	priv_data = g_object_get_data (G_OBJECT (cnc), OBJECT_DATA_ORACLE_HANDLE);
 	if (!priv_data) {
-		gda_connection_add_error_string (cnc, _("Invalid Oracle handle"));
+		gda_connection_add_event_string (cnc, _("Invalid Oracle handle"));
 		return FALSE;
 	}
 
@@ -826,6 +827,7 @@ gda_oracle_provider_execute_command (GdaServerProvider *provider,
 		g_free (str);
 		break;
 	default:
+		break;
 	}
 	/* don't know what to do here yet. */
 	return reclist;
@@ -849,12 +851,12 @@ gda_oracle_provider_begin_transaction (GdaServerProvider *provider,
 
 	priv_data = g_object_get_data (G_OBJECT (cnc), OBJECT_DATA_ORACLE_HANDLE);
 	if (!priv_data) {
-		gda_connection_add_error_string (cnc, _("Invalid Oracle handle"));
+		gda_connection_add_event_string (cnc, _("Invalid Oracle handle"));
 		return FALSE;
 	}
 
 	if (gda_connection_get_options (cnc) & GDA_CONNECTION_OPTIONS_READ_ONLY) {
-		gda_connection_add_error_string (cnc, _("Transactions are not supported in read-only mode"));
+		gda_connection_add_event_string (cnc, _("Transactions are not supported in read-only mode"));
 		return FALSE;
 	}
 
@@ -931,12 +933,12 @@ gda_oracle_provider_commit_transaction (GdaServerProvider *provider,
 	priv_data = g_object_get_data (G_OBJECT (cnc), OBJECT_DATA_ORACLE_HANDLE);
 	ora_xaction = g_object_get_data (G_OBJECT (xaction), OBJECT_DATA_ORACLE_HANDLE);
 	if (!priv_data || !ora_xaction) {
-		gda_connection_add_error_string (cnc, _("Invalid Oracle handle"));
+		gda_connection_add_event_string (cnc, _("Invalid Oracle handle"));
 		return FALSE;
 	}
 
 	if (gda_connection_get_options (cnc) & GDA_CONNECTION_OPTIONS_READ_ONLY) {
-		gda_connection_add_error_string (cnc, _("Transactions are not supported in read-only mode"));
+		gda_connection_add_event_string (cnc, _("Transactions are not supported in read-only mode"));
 		return FALSE;
 	}
 
@@ -992,12 +994,12 @@ gda_oracle_provider_rollback_transaction (GdaServerProvider *provider,
 	priv_data = g_object_get_data (G_OBJECT (cnc), OBJECT_DATA_ORACLE_HANDLE);
 	ora_xaction = g_object_get_data (G_OBJECT (xaction), OBJECT_DATA_ORACLE_HANDLE);
 	if (!priv_data || !ora_xaction) {
-		gda_connection_add_error_string (cnc, _("Invalid Oracle handle"));
+		gda_connection_add_event_string (cnc, _("Invalid Oracle handle"));
 		return FALSE;
 	}
 
 	if (gda_connection_get_options (cnc) & GDA_CONNECTION_OPTIONS_READ_ONLY) {
-		gda_connection_add_error_string (cnc, _("Transactions are not supported in read-only mode"));
+		gda_connection_add_event_string (cnc, _("Transactions are not supported in read-only mode"));
 		return FALSE;
 	}
 
@@ -1045,6 +1047,7 @@ gda_oracle_provider_supports (GdaServerProvider *provider,
 	case GDA_CONNECTION_FEATURE_VIEWS :
 		return TRUE;
 	default :
+		break;
 	}
 
 	return FALSE;
@@ -1139,7 +1142,7 @@ get_oracle_aggregates (GdaConnection *cnc, GdaParameterList *params)
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
 	priv_data = g_object_get_data (G_OBJECT (cnc), OBJECT_DATA_ORACLE_HANDLE);
 	if (!priv_data) {
-		gda_connection_add_error_string (cnc, _("Invalid Oracle handle"));
+		gda_connection_add_event_string (cnc, _("Invalid Oracle handle"));
 		return NULL;
 	}
 
@@ -1756,7 +1759,7 @@ gda_oracle_fill_md_data (const gchar *tblname,
 			return NULL;
 		}
 
-		value = gda_value_new_boolean (nullable > 0);
+		value = gda_value_new_boolean (! (nullable > 0));
 		rowlist = g_list_append (rowlist, value);
 
 		if (!h_table_index) {
@@ -2061,24 +2064,29 @@ get_oracle_objects (GdaConnection *cnc, GdaParameterList *params, GdaConnectionS
 
 typedef struct
 {
-    const gchar  *name;
-    GdaValueType type;
+	const gchar  *name;
+	GdaValueType  type;
+	const gchar  *synonyms;
 } ora_native_type;
 
 static const ora_native_type ora_type_tab[] =
 {
-	{ "BFILE",     GDA_VALUE_TYPE_BINARY    },
-	{ "BLOB",      GDA_VALUE_TYPE_BINARY    },
-	{ "CHAR",      GDA_VALUE_TYPE_STRING    },
-	{ "CLOB",      GDA_VALUE_TYPE_STRING    },
-	{ "DATE",      GDA_VALUE_TYPE_TIMESTAMP },
-	{ "NUMBER",    GDA_VALUE_TYPE_NUMERIC   },
-	{ "LONG",      GDA_VALUE_TYPE_STRING    },
-	{ "LONG RAW",  GDA_VALUE_TYPE_BINARY    },
-	{ "RAW",       GDA_VALUE_TYPE_BINARY    },
-	{ "ROWID",     GDA_VALUE_TYPE_STRING    },
-	{ "TIMESTAMP", GDA_VALUE_TYPE_TIMESTAMP },
-	{ "VARCHAR2",  GDA_VALUE_TYPE_STRING    }
+	{ "BFILE",     GDA_VALUE_TYPE_BINARY, NULL   },
+	{ "BLOB",      GDA_VALUE_TYPE_BINARY, NULL   },
+	{ "CHAR",      GDA_VALUE_TYPE_STRING, "CHARACTER"   },
+	{ "NCHAR",     GDA_VALUE_TYPE_STRING, NULL   },
+	{ "CLOB",      GDA_VALUE_TYPE_STRING, NULL   },
+	{ "NCLOB",     GDA_VALUE_TYPE_STRING, NULL   },
+	{ "DATE",      GDA_VALUE_TYPE_TIMESTAMP, NULL},
+	{ "NUMBER",    GDA_VALUE_TYPE_NUMERIC, "INTEGER,SMALLINT,INT,DEC,DECIMAL,NUMERIC,DOUBLE PRECISION,FLOAT,REAL"  },
+	{ "LONG",      GDA_VALUE_TYPE_STRING, NULL   },
+	{ "LONG RAW",  GDA_VALUE_TYPE_BINARY, NULL   },
+	{ "RAW",       GDA_VALUE_TYPE_BINARY, NULL   },
+	{ "ROWID",     GDA_VALUE_TYPE_STRING, NULL   },
+	{ "UROWID",    GDA_VALUE_TYPE_STRING, NULL   },
+	{ "TIMESTAMP", GDA_VALUE_TYPE_TIMESTAMP, NULL},
+	{ "VARCHAR2",  GDA_VALUE_TYPE_STRING, "VARCHAR,STRING"   },
+	{ "NVARCHAR2", GDA_VALUE_TYPE_STRING, NULL },
 };
 static const ora_native_type *
 ora_type_end = ora_type_tab+sizeof(ora_type_tab)/sizeof(ora_native_type);
@@ -2091,10 +2099,13 @@ get_oracle_types (GdaConnection *cnc, GdaParameterList *params)
     GList             *value_list;
     
     /* create the recordset */
-    recset = GDA_DATA_MODEL_ARRAY (gda_data_model_array_new (4));
+    recset = GDA_DATA_MODEL_ARRAY (gda_data_model_array_new (5));
     gda_data_model_set_column_title (GDA_DATA_MODEL (recset), 0, _("Type"));
     gda_data_model_set_column_title (GDA_DATA_MODEL (recset), 1, _("Owner"));
-    gda_data_model_set_column_title (GDA_DATA_MODEL (recset), 2, _("Comments"));    gda_data_model_set_column_title (GDA_DATA_MODEL (recset), 3, _("GDA type"));
+    gda_data_model_set_column_title (GDA_DATA_MODEL (recset), 2, _("Comments"));    
+    gda_data_model_set_column_title (GDA_DATA_MODEL (recset), 3, _("GDA type"));
+    gda_data_model_set_column_title (GDA_DATA_MODEL (recset), 4, _("Synonyms"));
+
     /* fill the recordset */
     for (otp = ora_type_tab; otp < ora_type_end; otp++)
     {
@@ -2102,7 +2113,8 @@ get_oracle_types (GdaConnection *cnc, GdaParameterList *params)
         value_list = g_list_append (value_list, gda_value_new_string (otp->name));
         value_list = g_list_append (value_list, gda_value_new_string ("SYS"));
         value_list = g_list_append (value_list, gda_value_new_string ("NULL"));
-        value_list = g_list_append (value_list, gda_value_new_type (otp->type));
+        value_list = g_list_append (value_list, gda_value_new_gdatype (otp->type));
+        value_list = g_list_append (value_list, gda_value_new_string (otp->synonyms));
 
 	gda_data_model_append_values (GDA_DATA_MODEL (recset), value_list);
         g_list_foreach (value_list, (GFunc) gda_value_free, NULL);
