@@ -1,8 +1,9 @@
 /* GDA Oracle provider
- * Copyright (C) 1998-2002 The GNOME Foundation.
+ * Copyright (C) 1998 - 2005 The GNOME Foundation.
  *
  * AUTHORS:
  * 	Tim Coleman <tim@timcoleman.com>
+ *      Vivien Malerba <malerba@gnome-db.org>
  *
  * This Library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public License as
@@ -25,9 +26,10 @@
 #include "gda-oracle.h"
 #include "gda-oracle-recordset.h"
 #include "gda-oracle-provider.h"
+#include <libgda/gda-data-model.h>
 #include <oci.h>
 
-#define PARENT_TYPE GDA_TYPE_DATA_MODEL
+#define PARENT_TYPE GDA_TYPE_DATA_MODEL_BASE
 
 struct _GdaOracleRecordsetPrivate {
 	GdaConnection *cnc;
@@ -59,6 +61,7 @@ define_columns (GdaOracleRecordsetPrivate *priv)
 	GList *fields = NULL;
 	gint i;
 	gint result;
+	ub2 fake_type;
 
 	for (i = 0; i < priv->ncolumns; i += 1) {
 		GdaOracleValue *ora_value = g_new0 (GdaOracleValue, 1);
@@ -97,19 +100,21 @@ define_columns (GdaOracleRecordsetPrivate *priv)
 			return NULL;
 		}
 
+		/* for data fetching  */
+		fake_type = ora_value->sql_type;
 		switch (ora_value->sql_type) {
-		case SQLT_NUM: /* Numerics are coerced to string */
-			ora_value->sql_type = SQLT_CHR;
-			break;
-		case SQLT_DAT: /* Convert SQLT_DAT to OCIDate */
-			ora_value->sql_type = SQLT_ODT;
-			break;
-		}
+                case SQLT_NUM: /* Numerics are coerced to string */
+                        fake_type = SQLT_CHR;
+                        break;
+                case SQLT_DAT: /* Convert SQLT_DAT to OCIDate */
+                        fake_type = SQLT_ODT;
+                        break;
+                }
 
 		if (ora_value->defined_size == 0)
-		    ora_value->value = NULL;
+			ora_value->value = NULL;
 		else
-		    ora_value->value = g_malloc0 (ora_value->defined_size+1);
+			ora_value->value = g_malloc0 (ora_value->defined_size+1);
 		ora_value->hdef = (OCIDefine *) 0;
 		ora_value->indicator = 0;
 		ora_value->gda_type = oracle_sqltype_to_gda_type (ora_value->sql_type);
@@ -119,7 +124,7 @@ define_columns (GdaOracleRecordsetPrivate *priv)
 					(ub4) i + 1,
 					ora_value->value,
 					ora_value->defined_size+1,
-					(ub2) ora_value->sql_type,
+					(ub2) fake_type,
 					(dvoid *) &(ora_value->indicator),
 					(ub2 *) 0,
 					(ub2 *) 0, 
@@ -140,12 +145,12 @@ define_columns (GdaOracleRecordsetPrivate *priv)
  * GdaOracleRecordset class implementation
  */
 
-static GdaFieldAttributes *
-gda_oracle_recordset_describe_column (GdaDataModel *model, gint col)
+static GdaColumn *
+gda_oracle_recordset_describe_column (GdaDataModelBase *model, gint col)
 {
 	GdaOracleRecordset *recset = (GdaOracleRecordset *) model;
 	GdaOracleRecordsetPrivate *priv_data;
-	GdaFieldAttributes *field_attrs;
+	GdaColumn *field_attrs;
 	OCIParam *pard;
 	gchar *pgchar_dummy;
 	glong col_name_len;
@@ -163,18 +168,18 @@ gda_oracle_recordset_describe_column (GdaDataModel *model, gint col)
 	priv_data = recset->priv;
 
 	if (!priv_data->cdata) {
-		gda_connection_add_error_string (priv_data->cnc, 
+		gda_connection_add_event_string (priv_data->cnc, 
 						_("Invalid Oracle handle"));
 		return NULL;
 	}
 
 	if (col >= priv_data->ncolumns) {
-		gda_connection_add_error_string (priv_data->cnc, 
+		gda_connection_add_event_string (priv_data->cnc, 
 						_("Column out of range"));
 		return NULL;
 	}
 
-	field_attrs = gda_field_attributes_new ();
+	field_attrs = gda_column_new ();
 
 	result = OCIParamGet ((dvoid *) priv_data->hstmt,
 				(ub4) OCI_HTYPE_STMT,
@@ -248,21 +253,21 @@ gda_oracle_recordset_describe_column (GdaDataModel *model, gint col)
 		return NULL;
 	}
 
-	gda_field_attributes_set_name (field_attrs, name_buffer);
-	gda_field_attributes_set_scale (field_attrs, scale);
-	gda_field_attributes_set_gdatype (field_attrs, oracle_sqltype_to_gda_type (sql_type));
-	gda_field_attributes_set_defined_size (field_attrs, defined_size);
+	gda_column_set_name (field_attrs, name_buffer);
+	gda_column_set_scale (field_attrs, scale);
+	gda_column_set_gdatype (field_attrs, oracle_sqltype_to_gda_type (sql_type));
+	gda_column_set_defined_size (field_attrs, defined_size);
 
 	/* FIXME */
-	gda_field_attributes_set_references (field_attrs, "");
+	gda_column_set_references (field_attrs, "");
 
 	/* FIXME */
-	gda_field_attributes_set_primary_key (field_attrs, FALSE); 
+	gda_column_set_primary_key (field_attrs, FALSE); 
 
 	/* FIXME */
-	gda_field_attributes_set_unique_key (field_attrs, FALSE);
+	gda_column_set_unique_key (field_attrs, FALSE);
 
-	gda_field_attributes_set_allow_null (field_attrs, !(nullable == 0));
+	gda_column_set_allow_null (field_attrs, !(nullable == 0));
 	
 	OCIDescriptorFree ((dvoid *) pard, OCI_DTYPE_PARAM);
 
@@ -270,7 +275,7 @@ gda_oracle_recordset_describe_column (GdaDataModel *model, gint col)
 }
 
 static gint
-gda_oracle_recordset_get_n_rows (GdaDataModel *model)
+gda_oracle_recordset_get_n_rows (GdaDataModelBase *model)
 {
 	GdaOracleRecordset *recset = (GdaOracleRecordset *) model;
 	GdaOracleRecordsetPrivate *priv_data;
@@ -317,7 +322,7 @@ gda_oracle_recordset_get_n_rows (GdaDataModel *model)
 }
 
 static gint
-gda_oracle_recordset_get_n_columns (GdaDataModel *model)
+gda_oracle_recordset_get_n_columns (GdaDataModelBase *model)
 {
 	GdaOracleRecordset *recset = (GdaOracleRecordset *) model;
 	g_return_val_if_fail (GDA_IS_ORACLE_RECORDSET (model), 0);
@@ -372,7 +377,7 @@ fetch_row (GdaOracleRecordset *recset, gint rownum)
 }
 
 static const GdaRow *
-gda_oracle_recordset_get_row (GdaDataModel *model, gint row)
+gda_oracle_recordset_get_row (GdaDataModelBase *model, gint row)
 {
 	GdaOracleRecordset *recset = (GdaOracleRecordset *) model;
 	GdaOracleRecordsetPrivate *priv_data;
@@ -386,7 +391,7 @@ gda_oracle_recordset_get_row (GdaDataModel *model, gint row)
 	priv_data = recset->priv;
 
 	if (!priv_data->cdata) {
-		gda_connection_add_error_string (priv_data->cnc,
+		gda_connection_add_event_string (priv_data->cnc,
 						_("Invalid Oracle handle"));
 		return NULL;
 	}
@@ -414,7 +419,7 @@ gda_oracle_recordset_get_row (GdaDataModel *model, gint row)
 }
 
 static const GdaValue *
-gda_oracle_recordset_get_value_at (GdaDataModel *model, gint col, gint row)
+gda_oracle_recordset_get_value_at (GdaDataModelBase *model, gint col, gint row)
 {
 	GdaOracleRecordset *recset = (GdaOracleRecordset *) model;
 	GdaOracleRecordsetPrivate *priv_data;
@@ -433,20 +438,20 @@ gda_oracle_recordset_get_value_at (GdaDataModel *model, gint col, gint row)
 }
 
 static gboolean
-gda_oracle_recordset_is_editable (GdaDataModel *model)
+gda_oracle_recordset_is_updatable (GdaDataModelBase *model)
 {
 	GdaCommandType cmd_type;
 	GdaOracleRecordset *recset = (GdaOracleRecordset *) model;
 
 	g_return_val_if_fail (GDA_IS_ORACLE_RECORDSET (recset), FALSE);
 
-	cmd_type = gda_data_model_get_command_type (model);
+	cmd_type = gda_data_model_get_command_type (GDA_DATA_MODEL (model));
 
 	return cmd_type == GDA_COMMAND_TYPE_TABLE ? TRUE : FALSE;
 }
 
 static const GdaRow *
-gda_oracle_recordset_append_row (GdaDataModel *model, const GList *values)
+gda_oracle_recordset_append_values (GdaDataModelBase *model, const GList *values)
 {
 	GString *sql;
 	GdaRow *row;
@@ -457,29 +462,29 @@ gda_oracle_recordset_append_row (GdaDataModel *model, const GList *values)
 
 	g_return_val_if_fail (GDA_IS_ORACLE_RECORDSET (recset), NULL);
 	g_return_val_if_fail (values != NULL, NULL);
-	g_return_val_if_fail (gda_data_model_is_editable (model), NULL);
-	g_return_val_if_fail (gda_data_model_is_editing (model), NULL);
+	g_return_val_if_fail (gda_data_model_is_updatable (GDA_DATA_MODEL (model)), NULL);
+	g_return_val_if_fail (gda_data_model_has_changed (GDA_DATA_MODEL (model)), NULL);
 	g_return_val_if_fail (recset->priv != NULL, 0);
 
 	priv_data = recset->priv;
 
 	if (priv_data->ncolumns != g_list_length ((GList *) values)) {
-		gda_connection_add_error_string (
+		gda_connection_add_event_string (
 			recset->priv->cnc,
 			_("Attempt to insert a row with an invalid number of columns"));
 		return NULL;
 	}
 
 	sql = g_string_new ("INSERT INTO ");
-	sql = g_string_append (sql, gda_data_model_get_command_text (model));
+	sql = g_string_append (sql, gda_data_model_get_command_text (GDA_DATA_MODEL (model)));
 	sql = g_string_append (sql, "(");
 
 	for (i = 0; i < priv_data->ncolumns; i += 1) {
-		GdaFieldAttributes *fa;
+		GdaColumn *fa;
 
-		fa = gda_data_model_describe_column (model, i);
+		fa = gda_data_model_describe_column (GDA_DATA_MODEL (model), i);
 		if (!fa) {
-			gda_connection_add_error_string (
+			gda_connection_add_event_string (
 				recset->priv->cnc,
 				_("Could not retrieve column's information"));
 			g_string_free (sql, TRUE);
@@ -488,7 +493,7 @@ gda_oracle_recordset_append_row (GdaDataModel *model, const GList *values)
 
 		if (i != 0) 
 			sql = g_string_append (sql, ", ");
-		sql = g_string_append (sql, gda_field_attributes_get_name (fa));
+		sql = g_string_append (sql, gda_column_get_name (fa));
 	}
 	sql = g_string_append (sql, ") VALUES (");
 
@@ -497,7 +502,7 @@ gda_oracle_recordset_append_row (GdaDataModel *model, const GList *values)
 		const GdaValue *val = (const GdaValue *) l->data;
 
 		if (!val) { 
-			gda_connection_add_error_string (
+			gda_connection_add_event_string (
 				recset->priv->cnc,
 				_("Could not retrieve column's value"));
 			g_string_free (sql, TRUE);
@@ -518,20 +523,20 @@ gda_oracle_recordset_append_row (GdaDataModel *model, const GList *values)
 
 	g_string_free (sql, TRUE);
 
-	row = gda_row_new_from_list (model, values);
+	row = gda_row_new_from_list (GDA_DATA_MODEL (model), values);
 	g_ptr_array_add (recset->priv->rows, row);
 
 	return (const GdaRow *) row;
 }
 
 static gboolean
-gda_oracle_recordset_remove_row (GdaDataModel *model, const GdaRow *row)
+gda_oracle_recordset_remove_row (GdaDataModelBase *model, const GdaRow *row)
 {
 	return FALSE;
 }
 
 static gboolean
-gda_oracle_recordset_update_row (GdaDataModel *model, const GdaRow *row)
+gda_oracle_recordset_update_row (GdaDataModelBase *model, const GdaRow *row)
 {
 	return FALSE;
 }
@@ -552,7 +557,7 @@ static void
 gda_oracle_recordset_class_init (GdaOracleRecordsetClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
-	GdaDataModelClass *model_class = GDA_DATA_MODEL_CLASS (klass);
+	GdaDataModelBaseClass *model_class = GDA_DATA_MODEL_BASE_CLASS (klass);
 
 	parent_class = g_type_class_peek_parent (klass);
 
@@ -562,8 +567,8 @@ gda_oracle_recordset_class_init (GdaOracleRecordsetClass *klass)
 	model_class->describe_column = gda_oracle_recordset_describe_column;
 	model_class->get_row = gda_oracle_recordset_get_row;
 	model_class->get_value_at = gda_oracle_recordset_get_value_at;
-	model_class->is_editable = gda_oracle_recordset_is_editable;
-	model_class->append_row = gda_oracle_recordset_append_row;
+	model_class->is_updatable = gda_oracle_recordset_is_updatable;
+	model_class->append_values = gda_oracle_recordset_append_values;
 	model_class->remove_row = gda_oracle_recordset_remove_row;
 	model_class->update_row = gda_oracle_recordset_update_row;
 }
@@ -587,9 +592,9 @@ gda_oracle_recordset_finalize (GObject *object)
 	priv_data = recset->priv;
 
 	if (!priv_data->cdata) {
-		gda_connection_add_error_string (priv_data->cnc, 
+		gda_connection_add_event_string (priv_data->cnc, 
 						_("Invalid Oracle handle"));
-		return NULL;
+		return;
 	}
 
 	g_return_if_fail (GDA_IS_ORACLE_RECORDSET (recset));
@@ -659,7 +664,7 @@ gda_oracle_recordset_new (GdaConnection *cnc,
 	recset->priv->hstmt = stmthp;
 	recset->priv->ncolumns = parcount;
 	recset->priv->ora_values = define_columns (recset->priv);
-	recset->priv->nrows = gda_oracle_recordset_get_n_rows (GDA_DATA_MODEL (recset));
+	recset->priv->nrows = gda_oracle_recordset_get_n_rows (recset);
 
 	return recset;
 }
